#
# This is an auto-generated file.  DO NOT EDIT!
#


from typing import Union, List, Tuple

class methods:
    fluent_name = ...
    child_names = ...
    p_v_coupling = ...
    flux_type = ...
    discretization_scheme = ...
    pseudo_time_method = ...
    unsteady_1st_order = ...
    unsteady_2nd_order = ...
    unsteady_2nd_order_bounded = ...
    unsteady_global_time = ...
    accelerated_non_iterative_time_marching = ...
    convergence_acceleration_for_stretched_meshes = ...
    expert = ...
    frozen_flux = ...
    gradient_scheme = ...
    high_order_term_relaxation = ...
    multiphase_numerics = ...
    nb_gradient_boundary_option = ...
    nita = ...
    nita_expert_controls = ...
    overset = ...
    phase_based_vof_discretization = ...
    reduced_rank_extrapolation = ...
    reduced_rank_extrapolation_options = ...
    residual_smoothing = ...
    vof_numerics = ...
    warped_face_gradient_correction = ...
    command_names = ...

    def set_solution_methods_to_default(self, ):
        """
        Set solution methods to default values.
        """




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .thin_film import thin_film as thin_film_cls
from .liquid_vof_factor import liquid_vof_factor as liquid_vof_factor_cls
class boiling_parameters(Group):
    """
    Multiphase boiling parameters menu.
    """

    fluent_name = "boiling-parameters"

    child_names = \
        ['thin_film', 'liquid_vof_factor']

    thin_film: thin_film_cls = thin_film_cls
    """
    thin_film child of boiling_parameters.
    """
    liquid_vof_factor: liquid_vof_factor_cls = liquid_vof_factor_cls
    """
    liquid_vof_factor child of boiling_parameters.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#


from typing import Union, List, Tuple

class histogram:
    fluent_name = ...
    command_names = ...

    def print(self, domain: str, cell_function: str, min_val: Union[float, str], max_val: Union[float, str], num_division: int, set_all_zones: bool, threads_list: List[str], file_name: str, overwrite: bool):
        """
        Print a histogram of a scalar quantity.
        
        Parameters
        ----------
            domain : str
                'domain' child.
            cell_function : str
                'cell_function' child.
            min_val : real
                'min_val' child.
            max_val : real
                'max_val' child.
            num_division : int
                'num_division' child.
            set_all_zones : bool
                'set_all_zones' child.
            threads_list : typing.List[str]
                'threads_list' child.
            file_name : str
                'file_name' child.
            overwrite : bool
                'overwrite' child.
        
        """

    def write(self, domain: str, cell_function: str, min_val: Union[float, str], max_val: Union[float, str], num_division: int, set_all_zones: bool, threads_list: List[str], file_name: str, overwrite: bool):
        """
        Write a histogram of a scalar quantity to a file.
        
        Parameters
        ----------
            domain : str
                'domain' child.
            cell_function : str
                'cell_function' child.
            min_val : real
                'min_val' child.
            max_val : real
                'max_val' child.
            num_division : int
                'num_division' child.
            set_all_zones : bool
                'set_all_zones' child.
            threads_list : typing.List[str]
                'threads_list' child.
            file_name : str
                'file_name' child.
            overwrite : bool
                'overwrite' child.
        
        """




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .option import option as option_cls
from .type_8 import type as type_cls
from .id import id as id_cls
from .normal import normal as normal_cls
from .partition import partition as partition_cls
class automatic(Group):
    """
    'automatic' child.
    """

    fluent_name = "automatic"

    child_names = \
        ['option', 'type', 'id', 'normal', 'partition']

    option: option_cls = option_cls
    """
    option child of automatic.
    """
    type: type_cls = type_cls
    """
    type child of automatic.
    """
    id: id_cls = id_cls
    """
    id child of automatic.
    """
    normal: normal_cls = normal_cls
    """
    normal child of automatic.
    """
    partition: partition_cls = partition_cls
    """
    partition child of automatic.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .oil_flow import oil_flow as oil_flow_cls
from .reverse import reverse as reverse_cls
from .node_values_1 import node_values as node_values_cls
from .relative_1 import relative as relative_cls
class options(Group):
    """
    'options' child.
    """

    fluent_name = "options"

    child_names = \
        ['oil_flow', 'reverse', 'node_values', 'relative']

    oil_flow: oil_flow_cls = oil_flow_cls
    """
    oil_flow child of options.
    """
    reverse: reverse_cls = reverse_cls
    """
    reverse child of options.
    """
    node_values: node_values_cls = node_values_cls
    """
    node_values child of options.
    """
    relative: relative_cls = relative_cls
    """
    relative child of options.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .zone_name import zone_name as zone_name_cls
from .value_1 import value as value_cls
class add_zone(Command):
    """
    'add_zone' command.
    
    Parameters
    ----------
        zone_name : str
            'zone_name' child.
        value : real
            'value' child.
    
    """

    fluent_name = "add-zone"

    argument_names = \
        ['zone_name', 'value']

    zone_name: zone_name_cls = zone_name_cls
    """
    zone_name argument of add_zone.
    """
    value: value_cls = value_cls
    """
    value argument of add_zone.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#


from typing import Union, List, Tuple

class mesh_interfaces:
    fluent_name = ...
    child_names = ...
    auto_options = ...
    interface = ...
    turbo_interface = ...
    enforce_continuity_after_bc = ...
    coupled_interfaces_inherit_bcs = ...
    verbosity = ...
    enable_si_with_nodes = ...
    enforce_coupled_wall_between_solids = ...
    enable_visualization_of_interfaces = ...
    mapped_interface_options = ...
    non_conformal_interface_numerics = ...
    command_names = ...

    def delete(self, name: str):
        """
        Delete a mesh interface.
        
        Parameters
        ----------
            name : str
                'name' child.
        
        """

    def display(self, zones: List[str]):
        """
        Display specified mesh interface zone.
        
        Parameters
        ----------
            zones : typing.List[str]
                Enter zone name list.
        
        """

    def list(self, ):
        """
        List all mesh-interfaces.
        """

    def make_phaselag_from_boundaries(self, side_1: str, side_2: str, angle: Union[float, str], interface_name: str):
        """
        Make interface zones phase lagged.
        
        Parameters
        ----------
            side_1 : str
                Enter id/name of zone to convert to phase lag side 1.
            side_2 : str
                Enter id/name of zone to convert to phase lag side 2.
            angle : real
                'angle' child.
            interface_name : str
                'interface_name' child.
        
        """

    def make_phaselag_from_periodic(self, periodic_zone_name: str):
        """
        Convert periodic interface to phase lagged.
        
        Parameters
        ----------
            periodic_zone_name : str
                Enter a periodic zone name.
        
        """

    def delete_interfaces_with_small_overlap(self, delete: bool, overlapping_percentage_threshold: Union[float, str]):
        """
        Delete mesh interfaces that have an area percentage under a specified value.
        
        Parameters
        ----------
            delete : bool
                'delete' child.
            overlapping_percentage_threshold : real
                'overlapping_percentage_threshold' child.
        
        """

    def delete_all(self, ):
        """
        Delete all mesh interfaces.
        """

    def improve_quality(self, check_mapped_interface_quality: bool, continue_: bool, tol_percentage_increment: Union[float, str]):
        """
        Improve mesh interface quality.
        
        Parameters
        ----------
            check_mapped_interface_quality : bool
                Check Mapped Interface Qaulity.
            continue_ : bool
                Continue to improve the mapped interface quality.
            tol_percentage_increment : real
                'tol_percentage_increment' child.
        
        """

    def one_to_one_pairing(self, o2o_flag: bool, toggle: bool, delete_empty: bool):
        """
        Use the default one-to-one interface creation method?.
        
        Parameters
        ----------
            o2o_flag : bool
                Use the default one-to-one interface creation method?.
            toggle : bool
                Would you like to proceed?.
            delete_empty : bool
                Delete empty interface interior zones from non-overlapping interfaces?.
        
        """

    def auto_pairing(self, all: bool, one_to_one_pairing: bool, new_interface_zones: List[str], si_create: bool, si_name: str, apply_mapped: bool, static_interface: bool):
        """
        Automatically pair and create mesh interfaces for some or all interface zones.
        
        Parameters
        ----------
            all : bool
                'all' child.
            one_to_one_pairing : bool
                'one_to_one_pairing' child.
            new_interface_zones : typing.List[str]
                Select unintersected interface zones for pairing.
            si_create : bool
                'si_create' child.
            si_name : str
                Enter a prefix for mesh interface names.
            apply_mapped : bool
                Apply Mapped option at solids.
            static_interface : bool
                'static_interface' child.
        
        """

    def transfer_motion_across_interfaces(self, enabled: bool, option_name: str):
        """
        Transfer motion from one side of the interface to the other when only one side undergoes user-defined or system-coupling motion.
        
        Parameters
        ----------
            enabled : bool
                'enabled' child.
            option_name : str
                'option_name' child.
        
        """

    def remove_left_handed_interface_faces(self, enable: bool, update: bool):
        """
        Remove left-handed faces during mesh interface creation.
        
        Parameters
        ----------
            enable : bool
                Remove left-handed faces on mesh interfaces.
            update : bool
                'update' child.
        
        """

    def non_overlapping_zone_name(self, zone_name: str):
        """
        Get non-overlapping zone name from the associated interface zone.
        
        Parameters
        ----------
            zone_name : str
                Enter zone id/name.
        
        """




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .general_nrbc import general_nrbc as general_nrbc_cls
from .turbo_sepcific_nrbc import turbo_sepcific_nrbc as turbo_sepcific_nrbc_cls
class non_reflecting_bc(Group):
    """
    'non_reflecting_bc' child.
    """

    fluent_name = "non-reflecting-bc"

    child_names = \
        ['general_nrbc', 'turbo_sepcific_nrbc']

    general_nrbc: general_nrbc_cls = general_nrbc_cls
    """
    general_nrbc child of non_reflecting_bc.
    """
    turbo_sepcific_nrbc: turbo_sepcific_nrbc_cls = turbo_sepcific_nrbc_cls
    """
    turbo_sepcific_nrbc child of non_reflecting_bc.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .surface_child import surface_child

class surface(NamedObject[surface_child], _CreatableNamedObjectMixin[surface_child]):
    """
    'surface' child.
    """

    fluent_name = "surface"

    child_object_type: surface_child = surface_child
    """
    child_object_type of surface.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .hide_environment_keep_effects import hide_environment_keep_effects as hide_environment_keep_effects_cls
from .environment_image import environment_image as environment_image_cls
from .vertical import vertical as vertical_cls
from .horizontal import horizontal as horizontal_cls
from .spin import spin as spin_cls
from .env_color import env_color as env_color_cls
from .env_intensity import env_intensity as env_intensity_cls
from .view_zoom import view_zoom as view_zoom_cls
from .show_backplate import show_backplate as show_backplate_cls
from .backplate_color import backplate_color as backplate_color_cls
from .backplate_image import backplate_image as backplate_image_cls
from .env_light_upvec import env_light_upvec as env_light_upvec_cls
from .env_light_dirvec import env_light_dirvec as env_light_dirvec_cls
class background(Group):
    """
    Enter the menu for background options.
    """

    fluent_name = "background"

    child_names = \
        ['hide_environment_keep_effects', 'environment_image', 'vertical',
         'horizontal', 'spin', 'env_color', 'env_intensity', 'view_zoom',
         'show_backplate', 'backplate_color', 'backplate_image',
         'env_light_upvec', 'env_light_dirvec']

    hide_environment_keep_effects: hide_environment_keep_effects_cls = hide_environment_keep_effects_cls
    """
    hide_environment_keep_effects child of background.
    """
    environment_image: environment_image_cls = environment_image_cls
    """
    environment_image child of background.
    """
    vertical: vertical_cls = vertical_cls
    """
    vertical child of background.
    """
    horizontal: horizontal_cls = horizontal_cls
    """
    horizontal child of background.
    """
    spin: spin_cls = spin_cls
    """
    spin child of background.
    """
    env_color: env_color_cls = env_color_cls
    """
    env_color child of background.
    """
    env_intensity: env_intensity_cls = env_intensity_cls
    """
    env_intensity child of background.
    """
    view_zoom: view_zoom_cls = view_zoom_cls
    """
    view_zoom child of background.
    """
    show_backplate: show_backplate_cls = show_backplate_cls
    """
    show_backplate child of background.
    """
    backplate_color: backplate_color_cls = backplate_color_cls
    """
    backplate_color child of background.
    """
    backplate_image: backplate_image_cls = backplate_image_cls
    """
    backplate_image child of background.
    """
    env_light_upvec: env_light_upvec_cls = env_light_upvec_cls
    """
    env_light_upvec child of background.
    """
    env_light_dirvec: env_light_dirvec_cls = env_light_dirvec_cls
    """
    env_light_dirvec child of background.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .pre_sweeps_2 import pre_sweeps as pre_sweeps_cls
from .post_sweeps_1 import post_sweeps as post_sweeps_cls
from .max_cycle_1 import max_cycle as max_cycle_cls
class fixed_cycle_parameters(Group):
    """
    'fixed_cycle_parameters' child.
    """

    fluent_name = "fixed-cycle-parameters"

    child_names = \
        ['pre_sweeps', 'post_sweeps', 'max_cycle']

    pre_sweeps: pre_sweeps_cls = pre_sweeps_cls
    """
    pre_sweeps child of fixed_cycle_parameters.
    """
    post_sweeps: post_sweeps_cls = post_sweeps_cls
    """
    post_sweeps child of fixed_cycle_parameters.
    """
    max_cycle: max_cycle_cls = max_cycle_cls
    """
    max_cycle child of fixed_cycle_parameters.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .active import active as active_cls
from .x_min import x_min as x_min_cls
from .y_min import y_min as y_min_cls
from .z_min import z_min as z_min_cls
from .x_max import x_max as x_max_cls
from .y_max import y_max as y_max_cls
from .z_max import z_max as z_max_cls
class settings_child(Group):
    """
    'child_object_type' of settings.
    """

    fluent_name = "child-object-type"

    child_names = \
        ['active', 'x_min', 'y_min', 'z_min', 'x_max', 'y_max', 'z_max']

    active: active_cls = active_cls
    """
    active child of settings_child.
    """
    x_min: x_min_cls = x_min_cls
    """
    x_min child of settings_child.
    """
    y_min: y_min_cls = y_min_cls
    """
    y_min child of settings_child.
    """
    z_min: z_min_cls = z_min_cls
    """
    z_min child of settings_child.
    """
    x_max: x_max_cls = x_max_cls
    """
    x_max child of settings_child.
    """
    y_max: y_max_cls = y_max_cls
    """
    y_max child of settings_child.
    """
    z_max: z_max_cls = z_max_cls
    """
    z_max child of settings_child.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .fensapice_flow_bc_subtype import fensapice_flow_bc_subtype as fensapice_flow_bc_subtype_cls
from .fensapice_drop_bccustom import fensapice_drop_bccustom as fensapice_drop_bccustom_cls
from .fensapice_drop_lwc import fensapice_drop_lwc as fensapice_drop_lwc_cls
from .fensapice_drop_dtemp import fensapice_drop_dtemp as fensapice_drop_dtemp_cls
from .fensapice_drop_ddiam import fensapice_drop_ddiam as fensapice_drop_ddiam_cls
from .fensapice_drop_dv import fensapice_drop_dv as fensapice_drop_dv_cls
from .fensapice_drop_dx import fensapice_drop_dx as fensapice_drop_dx_cls
from .fensapice_drop_dy import fensapice_drop_dy as fensapice_drop_dy_cls
from .fensapice_drop_dz import fensapice_drop_dz as fensapice_drop_dz_cls
from .fensapice_dpm_surface_injection import fensapice_dpm_surface_injection as fensapice_dpm_surface_injection_cls
from .fensapice_dpm_inj_nstream import fensapice_dpm_inj_nstream as fensapice_dpm_inj_nstream_cls
from .fensapice_drop_icc import fensapice_drop_icc as fensapice_drop_icc_cls
from .fensapice_drop_ctemp import fensapice_drop_ctemp as fensapice_drop_ctemp_cls
from .fensapice_drop_cmelt import fensapice_drop_cmelt as fensapice_drop_cmelt_cls
from .fensapice_drop_cdiam import fensapice_drop_cdiam as fensapice_drop_cdiam_cls
from .fensapice_drop_cv import fensapice_drop_cv as fensapice_drop_cv_cls
from .fensapice_drop_cx import fensapice_drop_cx as fensapice_drop_cx_cls
from .fensapice_drop_cy import fensapice_drop_cy as fensapice_drop_cy_cls
from .fensapice_drop_cz import fensapice_drop_cz as fensapice_drop_cz_cls
from .fensapice_drop_vrh import fensapice_drop_vrh as fensapice_drop_vrh_cls
from .fensapice_drop_vrh_1 import fensapice_drop_vrh_1 as fensapice_drop_vrh_1_cls
from .fensapice_drop_vc import fensapice_drop_vc as fensapice_drop_vc_cls
class icing(Group):
    """
    Help not available.
    """

    fluent_name = "icing"

    child_names = \
        ['fensapice_flow_bc_subtype', 'fensapice_drop_bccustom',
         'fensapice_drop_lwc', 'fensapice_drop_dtemp', 'fensapice_drop_ddiam',
         'fensapice_drop_dv', 'fensapice_drop_dx', 'fensapice_drop_dy',
         'fensapice_drop_dz', 'fensapice_dpm_surface_injection',
         'fensapice_dpm_inj_nstream', 'fensapice_drop_icc',
         'fensapice_drop_ctemp', 'fensapice_drop_cmelt',
         'fensapice_drop_cdiam', 'fensapice_drop_cv', 'fensapice_drop_cx',
         'fensapice_drop_cy', 'fensapice_drop_cz', 'fensapice_drop_vrh',
         'fensapice_drop_vrh_1', 'fensapice_drop_vc']

    fensapice_flow_bc_subtype: fensapice_flow_bc_subtype_cls = fensapice_flow_bc_subtype_cls
    """
    fensapice_flow_bc_subtype child of icing.
    """
    fensapice_drop_bccustom: fensapice_drop_bccustom_cls = fensapice_drop_bccustom_cls
    """
    fensapice_drop_bccustom child of icing.
    """
    fensapice_drop_lwc: fensapice_drop_lwc_cls = fensapice_drop_lwc_cls
    """
    fensapice_drop_lwc child of icing.
    """
    fensapice_drop_dtemp: fensapice_drop_dtemp_cls = fensapice_drop_dtemp_cls
    """
    fensapice_drop_dtemp child of icing.
    """
    fensapice_drop_ddiam: fensapice_drop_ddiam_cls = fensapice_drop_ddiam_cls
    """
    fensapice_drop_ddiam child of icing.
    """
    fensapice_drop_dv: fensapice_drop_dv_cls = fensapice_drop_dv_cls
    """
    fensapice_drop_dv child of icing.
    """
    fensapice_drop_dx: fensapice_drop_dx_cls = fensapice_drop_dx_cls
    """
    fensapice_drop_dx child of icing.
    """
    fensapice_drop_dy: fensapice_drop_dy_cls = fensapice_drop_dy_cls
    """
    fensapice_drop_dy child of icing.
    """
    fensapice_drop_dz: fensapice_drop_dz_cls = fensapice_drop_dz_cls
    """
    fensapice_drop_dz child of icing.
    """
    fensapice_dpm_surface_injection: fensapice_dpm_surface_injection_cls = fensapice_dpm_surface_injection_cls
    """
    fensapice_dpm_surface_injection child of icing.
    """
    fensapice_dpm_inj_nstream: fensapice_dpm_inj_nstream_cls = fensapice_dpm_inj_nstream_cls
    """
    fensapice_dpm_inj_nstream child of icing.
    """
    fensapice_drop_icc: fensapice_drop_icc_cls = fensapice_drop_icc_cls
    """
    fensapice_drop_icc child of icing.
    """
    fensapice_drop_ctemp: fensapice_drop_ctemp_cls = fensapice_drop_ctemp_cls
    """
    fensapice_drop_ctemp child of icing.
    """
    fensapice_drop_cmelt: fensapice_drop_cmelt_cls = fensapice_drop_cmelt_cls
    """
    fensapice_drop_cmelt child of icing.
    """
    fensapice_drop_cdiam: fensapice_drop_cdiam_cls = fensapice_drop_cdiam_cls
    """
    fensapice_drop_cdiam child of icing.
    """
    fensapice_drop_cv: fensapice_drop_cv_cls = fensapice_drop_cv_cls
    """
    fensapice_drop_cv child of icing.
    """
    fensapice_drop_cx: fensapice_drop_cx_cls = fensapice_drop_cx_cls
    """
    fensapice_drop_cx child of icing.
    """
    fensapice_drop_cy: fensapice_drop_cy_cls = fensapice_drop_cy_cls
    """
    fensapice_drop_cy child of icing.
    """
    fensapice_drop_cz: fensapice_drop_cz_cls = fensapice_drop_cz_cls
    """
    fensapice_drop_cz child of icing.
    """
    fensapice_drop_vrh: fensapice_drop_vrh_cls = fensapice_drop_vrh_cls
    """
    fensapice_drop_vrh child of icing.
    """
    fensapice_drop_vrh_1: fensapice_drop_vrh_1_cls = fensapice_drop_vrh_1_cls
    """
    fensapice_drop_vrh_1 child of icing.
    """
    fensapice_drop_vc: fensapice_drop_vc_cls = fensapice_drop_vc_cls
    """
    fensapice_drop_vc child of icing.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .sc_def_file_settings import sc_def_file_settings as sc_def_file_settings_cls
from .settings import settings as settings_cls
from .abaqus import abaqus as abaqus_cls
from .mechanical_apdl import mechanical_apdl as mechanical_apdl_cls
from .mechanical_apdl_input import mechanical_apdl_input as mechanical_apdl_input_cls
from .custom_heat_flux import custom_heat_flux as custom_heat_flux_cls
from .icemcfd_for_icepak import icemcfd_for_icepak as icemcfd_for_icepak_cls
from .fast_mesh import fast_mesh as fast_mesh_cls
from .fast_solution import fast_solution as fast_solution_cls
from .fast_velocity import fast_velocity as fast_velocity_cls
from .taitherm import taitherm as taitherm_cls
class export(Group):
    """
    'export' child.
    """

    fluent_name = "export"

    child_names = \
        ['sc_def_file_settings', 'settings']

    sc_def_file_settings: sc_def_file_settings_cls = sc_def_file_settings_cls
    """
    sc_def_file_settings child of export.
    """
    settings: settings_cls = settings_cls
    """
    settings child of export.
    """
    command_names = \
        ['abaqus', 'mechanical_apdl', 'mechanical_apdl_input',
         'custom_heat_flux', 'icemcfd_for_icepak', 'fast_mesh',
         'fast_solution', 'fast_velocity', 'taitherm']

    abaqus: abaqus_cls = abaqus_cls
    """
    abaqus command of export.
    """
    mechanical_apdl: mechanical_apdl_cls = mechanical_apdl_cls
    """
    mechanical_apdl command of export.
    """
    mechanical_apdl_input: mechanical_apdl_input_cls = mechanical_apdl_input_cls
    """
    mechanical_apdl_input command of export.
    """
    custom_heat_flux: custom_heat_flux_cls = custom_heat_flux_cls
    """
    custom_heat_flux command of export.
    """
    icemcfd_for_icepak: icemcfd_for_icepak_cls = icemcfd_for_icepak_cls
    """
    icemcfd_for_icepak command of export.
    """
    fast_mesh: fast_mesh_cls = fast_mesh_cls
    """
    fast_mesh command of export.
    """
    fast_solution: fast_solution_cls = fast_solution_cls
    """
    fast_solution command of export.
    """
    fast_velocity: fast_velocity_cls = fast_velocity_cls
    """
    fast_velocity command of export.
    """
    taitherm: taitherm_cls = taitherm_cls
    """
    taitherm command of export.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .list import list as list_cls
from .use_active import use_active as use_active_cls
from .restore_state import restore_state as restore_state_cls
from .copy_4 import copy as copy_cls
from .read_3 import read as read_cls
from .write_1 import write as write_cls
from .display_states_child import display_states_child

class display_states(NamedObject[display_states_child], _CreatableNamedObjectMixin[display_states_child]):
    """
    'display_states' child.
    """

    fluent_name = "display-states"

    command_names = \
        ['list', 'use_active', 'restore_state', 'copy', 'read', 'write']

    list: list_cls = list_cls
    """
    list command of display_states.
    """
    use_active: use_active_cls = use_active_cls
    """
    use_active command of display_states.
    """
    restore_state: restore_state_cls = restore_state_cls
    """
    restore_state command of display_states.
    """
    copy: copy_cls = copy_cls
    """
    copy command of display_states.
    """
    read: read_cls = read_cls
    """
    read command of display_states.
    """
    write: write_cls = write_cls
    """
    write command of display_states.
    """
    child_object_type: display_states_child = display_states_child
    """
    child_object_type of display_states.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .track_zone import track_zone as track_zone_cls
class zone_track(Group):
    """
    'zone_track' child.
    """

    fluent_name = "zone-track"

    child_names = \
        ['track_zone']

    track_zone: track_zone_cls = track_zone_cls
    """
    track_zone child of zone_track.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#


from typing import Union, List, Tuple

class solve:
    fluent_name = ...
    command_names = ...

    def iterate(self, iter_count: int, time_steps_count: int, fluid_time_step_count: int, iter_per_time_step_count: int):
        """
        Iteration the multidomain conjugate heat transfer.
        
        Parameters
        ----------
            iter_count : int
                'iter_count' child.
            time_steps_count : int
                'time_steps_count' child.
            fluid_time_step_count : int
                'fluid_time_step_count' child.
            iter_per_time_step_count : int
                'iter_per_time_step_count' child.
        
        """

    def dual_time_iterate(self, iter_count: int, time_steps_count: int, fluid_time_step_count: int, iter_per_time_step_count: int):
        """
        Dual-time iterate the multidomain conjugate heat transfer.
        
        Parameters
        ----------
            iter_count : int
                'iter_count' child.
            time_steps_count : int
                'time_steps_count' child.
            fluid_time_step_count : int
                'fluid_time_step_count' child.
            iter_per_time_step_count : int
                'iter_per_time_step_count' child.
        
        """




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .direct_solar_irradiation import direct_solar_irradiation as direct_solar_irradiation_cls
from .diffuse_solar_irradiation import diffuse_solar_irradiation as diffuse_solar_irradiation_cls
from .spectral_fraction import spectral_fraction as spectral_fraction_cls
class illumination_parameters(Group):
    """
    'illumination_parameters' child.
    """

    fluent_name = "illumination-parameters"

    child_names = \
        ['direct_solar_irradiation', 'diffuse_solar_irradiation',
         'spectral_fraction']

    direct_solar_irradiation: direct_solar_irradiation_cls = direct_solar_irradiation_cls
    """
    direct_solar_irradiation child of illumination_parameters.
    """
    diffuse_solar_irradiation: diffuse_solar_irradiation_cls = diffuse_solar_irradiation_cls
    """
    diffuse_solar_irradiation child of illumination_parameters.
    """
    spectral_fraction: spectral_fraction_cls = spectral_fraction_cls
    """
    spectral_fraction child of illumination_parameters.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .use_vapor_species_heat_capacity import use_vapor_species_heat_capacity as use_vapor_species_heat_capacity_cls
class diffusion_controlled(Group):
    """
    'diffusion_controlled' child.
    """

    fluent_name = "diffusion-controlled"

    child_names = \
        ['use_vapor_species_heat_capacity']

    use_vapor_species_heat_capacity: use_vapor_species_heat_capacity_cls = use_vapor_species_heat_capacity_cls
    """
    use_vapor_species_heat_capacity child of diffusion_controlled.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .option_2 import option as option_cls
from .c1 import c1 as c1_cls
from .c2 import c2 as c2_cls
from .reference_viscosity import reference_viscosity as reference_viscosity_cls
from .reference_temperature import reference_temperature as reference_temperature_cls
from .effective_temperature import effective_temperature as effective_temperature_cls
class sutherland(Group):
    """
    'sutherland' child.
    """

    fluent_name = "sutherland"

    child_names = \
        ['option', 'c1', 'c2', 'reference_viscosity', 'reference_temperature',
         'effective_temperature']

    option: option_cls = option_cls
    """
    option child of sutherland.
    """
    c1: c1_cls = c1_cls
    """
    c1 child of sutherland.
    """
    c2: c2_cls = c2_cls
    """
    c2 child of sutherland.
    """
    reference_viscosity: reference_viscosity_cls = reference_viscosity_cls
    """
    reference_viscosity child of sutherland.
    """
    reference_temperature: reference_temperature_cls = reference_temperature_cls
    """
    reference_temperature child of sutherland.
    """
    effective_temperature: effective_temperature_cls = effective_temperature_cls
    """
    effective_temperature child of sutherland.
    """



import pytest
from util.meshing_workflow import new_mesh_session  # noqa: F401
from util.solver_workflow import new_solver_session  # noqa: F401


@pytest.mark.codegen_required
def test_solver_preferences(new_solver_session):
    solver = new_solver_session
    preferred_meshing = solver.preferences.MeshingWorkflow
    preferred_meshing.Verbosity = "off"
    assert preferred_meshing.Verbosity() == "off"

    preferred_meshing.CheckpointingOption = "Write into memory"
    assert preferred_meshing.CheckpointingOption() == "Write into memory"

    preferred_drawing = preferred_meshing.DrawSettings
    preferred_drawing.FacetLimit = 6000000
    assert preferred_drawing.FacetLimit() == 6000000
    preferred_drawing.FaceZoneLimit = 15000
    preferred_drawing.FaceZoneLimit() == 15000

    ansys_logo = solver.preferences.Appearance.AnsysLogo
    ansys_logo.Color = "white"
    assert ansys_logo.Color() == "white"

    ansys_logo.Color = "black"
    assert ansys_logo.Color() == "black"

    ansys_logo.Visible = True
    assert ansys_logo.Visible() is True

    perfered_graphics = solver.preferences.Graphics
    perfered_graphics.AnimationOption = "wireframe"
    assert perfered_graphics.AnimationOption() == "wireframe"

    solver.exit()


@pytest.mark.codegen_required
def test_meshing_preferences(new_mesh_session):
    meshing = new_mesh_session
    preferred_meshing = meshing.preferences.MeshingWorkflow
    preferred_meshing.Verbosity = "off"
    assert preferred_meshing.Verbosity() == "off"

    preferred_meshing.CheckpointingOption = "Write into memory"
    assert preferred_meshing.CheckpointingOption() == "Write into memory"

    preferred_drawing = preferred_meshing.DrawSettings
    preferred_drawing.FacetLimit = 6000000
    assert preferred_drawing.FacetLimit() == 6000000

    preferred_drawing.FaceZoneLimit = 15000
    preferred_drawing.FaceZoneLimit() == 15000

    ansys_logo = meshing.preferences.Appearance.AnsysLogo
    ansys_logo.Color = "white"
    assert ansys_logo.Color() == "white"

    ansys_logo.Color = "black"
    assert ansys_logo.Color() == "black"

    ansys_logo.Visible = True
    assert ansys_logo.Visible() is True

    preferred_graphics = meshing.preferences.Graphics
    preferred_graphics.AnimationOption = "wireframe"
    assert preferred_graphics.AnimationOption() == "wireframe"

    meshing.exit()



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .udf_zmotion_name import udf_zmotion_name as udf_zmotion_name_cls
from .axis_direction_1 import axis_direction as axis_direction_cls
from .axis_origin_1 import axis_origin as axis_origin_cls
from .cursys import cursys as cursys_cls
from .conical import conical as conical_cls
from .solid_omega import solid_omega as solid_omega_cls
from .solid_relative_to_thread import solid_relative_to_thread as solid_relative_to_thread_cls
from .solid_motion import solid_motion as solid_motion_cls
from .solid_motion_axis_direction import solid_motion_axis_direction as solid_motion_axis_direction_cls
from .solid_motion_axis_origin import solid_motion_axis_origin as solid_motion_axis_origin_cls
from .solid_motion_velocity import solid_motion_velocity as solid_motion_velocity_cls
from .solid_motion_zone_motion_function import solid_motion_zone_motion_function as solid_motion_zone_motion_function_cls
from .omega import omega as omega_cls
from .relative_to_thread import relative_to_thread as relative_to_thread_cls
from .motion_spec import motion_spec as motion_spec_cls
class disabled(Group):
    """
    Help not available.
    """

    fluent_name = "disabled"

    child_names = \
        ['udf_zmotion_name', 'axis_direction', 'axis_origin', 'cursys',
         'conical', 'solid_omega', 'solid_relative_to_thread', 'solid_motion',
         'solid_motion_axis_direction', 'solid_motion_axis_origin',
         'solid_motion_velocity', 'solid_motion_zone_motion_function',
         'omega', 'relative_to_thread', 'motion_spec']

    udf_zmotion_name: udf_zmotion_name_cls = udf_zmotion_name_cls
    """
    udf_zmotion_name child of disabled.
    """
    axis_direction: axis_direction_cls = axis_direction_cls
    """
    axis_direction child of disabled.
    """
    axis_origin: axis_origin_cls = axis_origin_cls
    """
    axis_origin child of disabled.
    """
    cursys: cursys_cls = cursys_cls
    """
    cursys child of disabled.
    """
    conical: conical_cls = conical_cls
    """
    conical child of disabled.
    """
    solid_omega: solid_omega_cls = solid_omega_cls
    """
    solid_omega child of disabled.
    """
    solid_relative_to_thread: solid_relative_to_thread_cls = solid_relative_to_thread_cls
    """
    solid_relative_to_thread child of disabled.
    """
    solid_motion: solid_motion_cls = solid_motion_cls
    """
    solid_motion child of disabled.
    """
    solid_motion_axis_direction: solid_motion_axis_direction_cls = solid_motion_axis_direction_cls
    """
    solid_motion_axis_direction child of disabled.
    """
    solid_motion_axis_origin: solid_motion_axis_origin_cls = solid_motion_axis_origin_cls
    """
    solid_motion_axis_origin child of disabled.
    """
    solid_motion_velocity: solid_motion_velocity_cls = solid_motion_velocity_cls
    """
    solid_motion_velocity child of disabled.
    """
    solid_motion_zone_motion_function: solid_motion_zone_motion_function_cls = solid_motion_zone_motion_function_cls
    """
    solid_motion_zone_motion_function child of disabled.
    """
    omega: omega_cls = omega_cls
    """
    omega child of disabled.
    """
    relative_to_thread: relative_to_thread_cls = relative_to_thread_cls
    """
    relative_to_thread child of disabled.
    """
    motion_spec: motion_spec_cls = motion_spec_cls
    """
    motion_spec child of disabled.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

class current_parametric_study(String):
    """
    Name of Current Parametric Study.
    """

    fluent_name = "current-parametric-study"




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .mass_flux_correction_method import mass_flux_correction_method as mass_flux_correction_method_cls
from .hybrid_mode_selection import hybrid_mode_selection as hybrid_mode_selection_cls
class expert(Group):
    """
    Enter overset expert solver options menu.
    """

    fluent_name = "expert"

    child_names = \
        ['mass_flux_correction_method', 'hybrid_mode_selection']

    mass_flux_correction_method: mass_flux_correction_method_cls = mass_flux_correction_method_cls
    """
    mass_flux_correction_method child of expert.
    """
    hybrid_mode_selection: hybrid_mode_selection_cls = hybrid_mode_selection_cls
    """
    hybrid_mode_selection child of expert.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .trim_option import trim_option as trim_option_cls
from .update_frequency import update_frequency as update_frequency_cls
from .damping_factor import damping_factor as damping_factor_cls
from .thrust_coef import thrust_coef as thrust_coef_cls
from .pitch_moment_coef import pitch_moment_coef as pitch_moment_coef_cls
from .roll_moment_coef import roll_moment_coef as roll_moment_coef_cls
class trimming(Group):
    """
    Menu to define rotor trimming set-up.
    
     - trim-option       : to define collective and cyclic pitches to trim, 
     - update-frequency  : the number of solver iterations that pitch angle will be updated each time, 
     - damping-factor    : relaxation factor for pitch angles, 
     - thrust-coef       : desired thrust coefficient to set pitch for
     - pitch-moment-coef : desired pitch-moment coefficient to set pitch for, 
     - roll-moment-coef  : desired roll-moment coefficient to set pitch for, 
    
    For more details please consult the help option of the corresponding menu or TUI command.
    """

    fluent_name = "trimming"

    child_names = \
        ['trim_option', 'update_frequency', 'damping_factor', 'thrust_coef',
         'pitch_moment_coef', 'roll_moment_coef']

    trim_option: trim_option_cls = trim_option_cls
    """
    trim_option child of trimming.
    """
    update_frequency: update_frequency_cls = update_frequency_cls
    """
    update_frequency child of trimming.
    """
    damping_factor: damping_factor_cls = damping_factor_cls
    """
    damping_factor child of trimming.
    """
    thrust_coef: thrust_coef_cls = thrust_coef_cls
    """
    thrust_coef child of trimming.
    """
    pitch_moment_coef: pitch_moment_coef_cls = pitch_moment_coef_cls
    """
    pitch_moment_coef child of trimming.
    """
    roll_moment_coef: roll_moment_coef_cls = roll_moment_coef_cls
    """
    roll_moment_coef child of trimming.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .verbosity_6 import verbosity as verbosity_cls
from .skewness_neighbor_coupling import skewness_neighbor_coupling as skewness_neighbor_coupling_cls
from .hybrid_nita_settings import hybrid_nita_settings as hybrid_nita_settings_cls
class nita_expert_controls(Group):
    """
    Enter the nita expert controls menu.
    """

    fluent_name = "nita-expert-controls"

    child_names = \
        ['verbosity', 'skewness_neighbor_coupling', 'hybrid_nita_settings']

    verbosity: verbosity_cls = verbosity_cls
    """
    verbosity child of nita_expert_controls.
    """
    skewness_neighbor_coupling: skewness_neighbor_coupling_cls = skewness_neighbor_coupling_cls
    """
    skewness_neighbor_coupling child of nita_expert_controls.
    """
    hybrid_nita_settings: hybrid_nita_settings_cls = hybrid_nita_settings_cls
    """
    hybrid_nita_settings child of nita_expert_controls.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

class isat_table_size(Real):
    """
    Specify the Max. Storage [Mb].
    """

    fluent_name = "isat-table-size"




#
# This is an auto-generated file.  DO NOT EDIT!
#


from typing import Union, List, Tuple

class number_of_bands:
    fluent_name = ...
    child_names = ...
    set_all_interfaces = ...
    command_names = ...

    def set_specific_interface(self, interface_number: int, bands: int):
        """
        Set number of band to be used for mixing.
        
        Parameters
        ----------
            interface_number : int
                Set number of band to be used for mixing.
            bands : int
                Set number of band to be used for mixing.
        
        """




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .custom_vector import custom_vector as custom_vector_cls
from .field import field as field_cls
from .surfaces import surfaces as surfaces_cls
from .geometry_1 import geometry as geometry_cls
from .physics import physics as physics_cls
from .retain_instantaneous_values import retain_instantaneous_values as retain_instantaneous_values_cls
from .report_type import report_type as report_type_cls
from .phase_25 import phase as phase_cls
from .average_over import average_over as average_over_cls
from .per_surface import per_surface as per_surface_cls
from .old_props import old_props as old_props_cls
from .surface_names import surface_names as surface_names_cls
from .surface_ids import surface_ids as surface_ids_cls
class surface_child(Group):
    """
    'child_object_type' of surface.
    """

    fluent_name = "child-object-type"

    child_names = \
        ['custom_vector', 'field', 'surfaces', 'geometry', 'physics',
         'retain_instantaneous_values', 'report_type', 'phase',
         'average_over', 'per_surface', 'old_props', 'surface_names',
         'surface_ids']

    custom_vector: custom_vector_cls = custom_vector_cls
    """
    custom_vector child of surface_child.
    """
    field: field_cls = field_cls
    """
    field child of surface_child.
    """
    surfaces: surfaces_cls = surfaces_cls
    """
    surfaces child of surface_child.
    """
    geometry: geometry_cls = geometry_cls
    """
    geometry child of surface_child.
    """
    physics: physics_cls = physics_cls
    """
    physics child of surface_child.
    """
    retain_instantaneous_values: retain_instantaneous_values_cls = retain_instantaneous_values_cls
    """
    retain_instantaneous_values child of surface_child.
    """
    report_type: report_type_cls = report_type_cls
    """
    report_type child of surface_child.
    """
    phase: phase_cls = phase_cls
    """
    phase child of surface_child.
    """
    average_over: average_over_cls = average_over_cls
    """
    average_over child of surface_child.
    """
    per_surface: per_surface_cls = per_surface_cls
    """
    per_surface child of surface_child.
    """
    old_props: old_props_cls = old_props_cls
    """
    old_props child of surface_child.
    """
    surface_names: surface_names_cls = surface_names_cls
    """
    surface_names child of surface_child.
    """
    surface_ids: surface_ids_cls = surface_ids_cls
    """
    surface_ids child of surface_child.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .list import list as list_cls
from .list_properties import list_properties as list_properties_cls
from .duplicate import duplicate as duplicate_cls
from .phase_child_17 import phase_child

class phase(NamedObject[phase_child], _NonCreatableNamedObjectMixin[phase_child]):
    """
    'phase' child.
    """

    fluent_name = "phase"

    command_names = \
        ['list', 'list_properties', 'duplicate']

    list: list_cls = list_cls
    """
    list command of phase.
    """
    list_properties: list_properties_cls = list_properties_cls
    """
    list_properties command of phase.
    """
    duplicate: duplicate_cls = duplicate_cls
    """
    duplicate command of phase.
    """
    child_object_type: phase_child = phase_child
    """
    child_object_type of phase.
    """



"""Workflow module that wraps and extends the core functionality."""

from __future__ import annotations

import logging
import threading
from typing import Any, Iterator, List, Optional, Tuple
import warnings

import ansys.fluent.core as pyfluent
from ansys.fluent.core.data_model_cache import DataModelCache
from ansys.fluent.core.services.datamodel_se import PyCallableStateObject, PyMenuGeneric

logger = logging.getLogger("pyfluent.datamodel")


def _new_command_for_task(task, session):
    class NewCommandError(Exception):
        def __init__(self, task_name):
            super().__init__(f"Could not create command for task {task_name}")

    task_cmd_name = task.CommandName()
    cmd_creator = getattr(session, task_cmd_name)
    if cmd_creator:
        new_cmd = cmd_creator.create_instance()
        if new_cmd:
            return new_cmd
    raise NewCommandError(task._name_())


def _init_task_accessors(obj):
    logger.debug("_init_task_accessors")
    logger.debug(f"thread ID in _init_task_accessors {threading.get_ident()}")
    for task in obj.ordered_children(recompute=True):
        py_name = task.python_name()
        logger.debug(f"py_name: {py_name}")
        with obj._lock:
            obj._python_task_names.append(py_name)
        if py_name not in obj._task_objects:
            logger.debug(f"adding {py_name} {type(task)}")
            obj._task_objects[py_name] = task
        else:
            logger.debug(
                f"Could not add task {py_name} {type(getattr(obj, py_name, None))}"
            )
        _init_task_accessors(task)


def _refresh_task_accessors(obj):
    logger.debug(f"thread ID in _refresh_task_accessors {threading.get_ident()}")
    with obj._lock:
        old_task_names = set(obj._python_task_names)
    logger.debug(f"_refresh_task_accessors old_task_names: {old_task_names}")
    tasks = obj.ordered_children(recompute=True)
    current_task_names = [task.python_name() for task in tasks]
    logger.debug(f"current_task_names: {current_task_names}")
    current_task_name_set = set(current_task_names)
    created_task_names = current_task_name_set - old_task_names
    deleted_task_names = old_task_names - current_task_name_set
    for task_name in deleted_task_names:
        try:
            del obj._task_objects[task_name]
        except KeyError:
            pass
    for task_name in created_task_names:
        if task_name not in obj._task_objects:
            logger.debug(f"Add task {task_name}")
            obj._task_objects[task_name] = tasks[current_task_names.index(task_name)]
        else:
            logger.debug(
                f"Could not add task {task_name} {type(getattr(obj, task_name, None))}"
            )
    with obj._lock:
        obj._python_task_names = current_task_names
        logger.debug(f"updated_task_names: {obj._python_task_names}")
    for task in tasks:
        logger.debug(f"next task {task.python_name()} {id(task)}")
        _refresh_task_accessors(task)


def _convert_task_list_to_display_names(workflow_root, task_list):
    if pyfluent.DATAMODEL_USE_STATE_CACHE:
        workflow_state = DataModelCache.get_state("workflow", workflow_root)
    else:
        workflow_state = workflow_root.get_remote_state()
    return [workflow_state[f"TaskObject:{x}"]["_name_"] for x in task_list]


class BaseTask:
    """Base class Task representation for wrapping a Workflow TaskObject instance,
    adding methods to discover more about the relationships between TaskObjects.

    Methods
    -------
    get_direct_upstream_tasks()
    get_direct_downstream_tasks()
    ordered_children()
    inactive_ordered_children()
    get_id()
    get_idx()
    __getattr__(attr)
    __setattr__(attr, value)
    __dir__()
    __call__()
    """

    def __init__(self, command_source: WorkflowWrapper, task: str) -> None:
        """Initialize BaseTask.

        Parameters
        ----------
        command_source : WorkflowWrapper
            The set of workflow commands.
        task : str
            The name of this task.
        """
        self.__dict__.update(
            dict(
                _command_source=command_source,
                _workflow=command_source._workflow,
                _source=command_source._command_source,
                _task=task,
                _cmd=None,
                _python_name=None,
                _python_task_names=[],
                _lock=command_source._lock,
                _ordered_children=[],
                _task_list=[],
                _task_objects={},
            )
        )

    def get_direct_upstream_tasks(self) -> list:
        """Get the list of tasks upstream of this one and directly connected by a data
        dependency.

        Returns
        -------
        list
            Upstream task list.
        """
        return self._tasks_with_matching_attributes(
            attr="requiredInputs", other_attr="outputs"
        )

    def get_direct_upstream_tasks(self) -> list:
        """Get the list of tasks upstream of this one and directly connected by a data
        dependency.

        Returns
        -------
        list
            Upstream task list.
        """
        return self._tasks_with_matching_attributes(
            attr="requiredInputs", other_attr="outputs"
        )

    def get_direct_downstream_tasks(self) -> list:
        """Get the list of tasks downstream of this one and directly connected by a data
        dependency.

        Returns
        -------
        list
            Downstream task list.
        """
        return self._tasks_with_matching_attributes(
            attr="outputs", other_attr="requiredInputs"
        )

    def ordered_children(self, recompute=True) -> list:
        """Get the ordered task list held by this task. Sorting is in terms of the
        workflow order and only includes this task's top-level tasks, while other tasks
        can be obtained by calling ordered_children() on a parent task.

        Given the workflow::

            Workflow
            ├── A
            ├── B
            │   ├── C
            │   └── D
            └── E

        C and D are the ordered children of task B.

        Returns
        -------
        list
            Ordered children.
        """
        if recompute:

            def task_by_id(mappings):
                def _task_by_id(task_id):
                    if task_id in mappings:
                        return mappings[task_id]
                    try:
                        return self._command_source._task_by_id(task_id)
                    except Exception:
                        pass

                return _task_by_id

            task_list = self._task.TaskList()
            task_list = _convert_task_list_to_display_names(self._workflow, task_list)
            if task_list != self._task_list:
                mappings = {
                    k: v for k, v in zip(self._task_list, self._ordered_children)
                }
                self._ordered_children = list(
                    filter(None, map(task_by_id(mappings), task_list))
                )
                self._task_list = task_list
        return self._ordered_children

    def inactive_ordered_children(self) -> list:
        """Get the inactive ordered child list.

        Returns
        -------
        list
            Inactive ordered children.
        """
        return []

    def child_task_python_names(self) -> List[str]:
        """Get the Pythonic names of the child tasks.

        Returns
        -------
        List[str]
            Pythonic names of the child tasks.
        """
        return self._python_task_names

    def get_id(self) -> str:
        """Get the unique string identifier of this task, as it is in the application.

        Returns
        -------
        str
            The string identifier.
        """
        workflow_state = self._command_source._workflow_state()
        for k, v in workflow_state.items():
            if isinstance(v, dict) and "_name_" in v:
                if v["_name_"] == self.name():
                    type_, id_ = k.split(":")
                    if type_ == "TaskObject":
                        return id_

    def get_idx(self) -> int:
        """Get the unique integer index of this task, as it is in the application.

        Returns
        -------
        int
            The integer index.
        """
        return int(self.get_id()[len("TaskObject") :])

    def python_name(self) -> str:
        """Get the Pythonic name of this task, as it is in the underlying application.

        Returns
        -------
        str
            The Pythonic name of this task.
        """
        if not self._python_name:
            try:
                this_command = self._command()
                # temp reuse helpString
                self._python_name = this_command.get_attr("helpString")
            except Exception:
                pass
        return self._python_name

    def delete(self) -> None:
        """Delete this task from the workflow."""
        self._command_source.DeleteTasks(ListOfTasks=[self.name()])

    def __getattr__(self, attr):
        try:
            result = getattr(self._task, attr)
            if result:
                return result
        except AttributeError:
            pass
        try:
            return ArgumentWrapper(self, attr)
        except Exception as ex:
            logger.debug(str(ex))
        return self._task_objects.get(attr, None)

    def __setattr__(self, attr, value):
        logger.debug(f"BaseTask.__setattr__({attr}, {value})")
        if attr in self.__dict__:
            self.__dict__[attr] = value
        else:
            setattr(self._task, attr, value)

    def __dir__(self):
        return sorted(
            set(list(self.__dict__.keys()) + dir(type(self)) + dir(self._task))
        )

    def __call__(self, **kwds) -> Any:
        if kwds:
            self._task.Arguments.set_state(**kwds)
        return self._task.Execute()

    def _tasks_with_matching_attributes(self, attr: str, other_attr: str) -> list:
        this_command = self._command()
        attrs = this_command.get_attr(attr)
        if not attrs:
            return []
        attrs = set(attrs)
        tasks = [
            task
            for task in self._command_source.ordered_children()
            if task.name() != self.name()
        ]
        matches = []
        for task in tasks:
            command = task._command()
            other_attrs = command.get_attr(other_attr)
            if other_attrs and (attrs & set(other_attrs)):
                matches.append(task)
        return matches

    def display_name(self):
        return self._name_()


class TaskContainer(PyCallableStateObject):
    """Wrap a workflow TaskObject container.

    Methods
    -------
    __iter__()
    __getitem__(attr)
    __getattr__(attr)
    __dir__()
    """

    def __init__(self, command_source: WorkflowWrapper) -> None:
        """Initialize TaskContainer.

        Parameters
        ----------
        command_source : WorkflowWrapper
            The set of workflow commands.
        """
        self._container = command_source
        self._task_container = command_source._workflow.TaskObject

    def __iter__(self) -> Iterator[BaseTask]:
        """Yield the next child object.

        Yields
        ------
        Iterator[BaseTask]
            Iterator of child objects.
        """
        for name in self._get_child_object_display_names():
            yield self[name]

    def __getitem__(self, name):
        logger.debug(f"TaskContainer.__getitem__({name})")
        return _makeTask(self._container, name)

    def __getattr__(self, attr):
        return getattr(self._task_container, attr)

    def __dir__(self):
        return sorted(
            set(
                list(self.__dict__.keys()) + dir(type(self)) + dir(self._task_container)
            )
        )

    def items(self):
        return self._task_container.get_state().items()

    def get_state(self):
        return self._task_container.get_state()

    def __call__(self):
        return self.get_state()


class ArgumentsWrapper(PyCallableStateObject):
    """Wrapper for a dictionary of task arguments."""

    def __init__(self, task: BaseTask) -> None:
        """Initialize ArgumentsWrapper.

        Parameters
        ----------
        task : BaseTask
            The task holding these arguments.
        """
        self._task = task

    def set_state(self, args: dict) -> None:
        """Overwrite arguments.

        Parameters
        ----------
        args : dict
            New argument state.
        """
        self._task.Arguments.set_state(args)

    def update_dict(self, args: dict) -> None:
        """Merge with arguments.

        Parameters
        ----------
        args : dict
            new arguments state
        """
        self._task.Arguments.update_dict(args)

    def get_state(self, explicit_only=False) -> dict:
        """Get arguments state.

        Parameters
        ----------
        explicit_only : bool
            Whether to only include explicitly set values,
            otherwise all values are included.
        """
        return (
            self._task.Arguments() if explicit_only else self._task._command_arguments()
        )

    def __getattr__(self, attr):
        return getattr(self._task._command_arguments, attr)

    def __setitem__(self, key, value):
        self._task._command_arguments.__setitem__(key, value)


class ArgumentWrapper(PyCallableStateObject):
    """Wrapper for a single task argument."""

    def __init__(self, task: BaseTask, arg: str) -> None:
        """Initialize ArgumentWrapper.

        Parameters
        ----------
        task : BaseTask
            The task holding these arguments.
        arg: str
            Argument name.
        """
        self._task = task
        self._arg_name = arg
        self._arg = getattr(task._command_arguments, arg)
        if self._arg is None:
            raise RuntimeError(f"{arg} is not an argument")

    def set_state(self, value: Any) -> None:
        """Set the state of this argument.

        Parameters
        ----------
        value : Any
            New argument value.
        """
        self._task.Arguments.update_dict({self._arg_name: value})

    def get_state(self, explicit_only: bool = False) -> Any:
        """Get argument state.

        Parameters
        ----------
        explicit_only : bool
            Whether to return the explicitly set value or the
            full derived value.
        """
        return self._task.Arguments()[self._arg_name] if explicit_only else self._arg()

    def __getattr__(self, attr):
        return getattr(self._arg, attr)


class CommandTask(BaseTask):
    """Intermediate base class task representation for wrapping a Workflow TaskObject
    instance, adding attributes related to commanding.

    Classes without these attributes cannot be commanded.
    """

    def __init__(self, command_source: WorkflowWrapper, task: str) -> None:
        """Initialize CommandTask.

        Parameters
        ----------
        command_source : WorkflowWrapper
            The set of workflow commands.
        task : str
            The name of this task.
        """
        super().__init__(command_source, task)

    @property
    def CommandArguments(self) -> ReadOnlyObject:
        """Get the task's arguments in read-only form (deprecated).

        Returns
        -------
        ReadOnlyObject
            The task's arguments.
        """
        warnings.warn("CommandArguments", DeprecationWarning)
        return self._refreshed_command()

    @property
    def _command_arguments(self) -> ReadOnlyObject:
        return self._refreshed_command()

    @property
    def arguments(self) -> ArgumentsWrapper:
        """Get the task's arguments.

        Returns
        -------
        ArgumentsWrapper
            The task's arguments.
        """
        return ArgumentsWrapper(self)

    def _refreshed_command(self) -> ReadOnlyObject:
        task_arg_state = self._task.Arguments.get_state()
        cmd = self._command()
        if task_arg_state:
            cmd.set_state(task_arg_state)
        return ReadOnlyObject(self._cmd_sub_items_read_only(cmd, cmd()))

    def _cmd_sub_items_read_only(self, cmd, cmd_state):
        for key, value in cmd_state.items():
            if type(value) == dict:
                setattr(
                    cmd, key, self._cmd_sub_items_read_only(getattr(cmd, key), value)
                )
            setattr(cmd, key, ReadOnlyObject(getattr(cmd, key)))
        return cmd

    def _command(self):
        if not self._cmd:
            self._cmd = _new_command_for_task(self._task, self._source)
        return self._cmd


class SimpleTask(CommandTask):
    """Simple task representation for wrapping a Workflow TaskObject instance of
    TaskType Simple."""

    def __init__(self, command_source: WorkflowWrapper, task: str) -> None:
        """Initialize SimpleTask.

        Parameters
        ----------
        command_source : WorkflowWrapper
            The set of workflow commands.
        task : str
            The name of this task.
        """
        super().__init__(command_source, task)

    def ordered_children(self, recompute=True) -> list:
        """Get the ordered task list held by the workflow.

        SimpleTasks have no TaskList.
        """
        return []


class CompoundChild(SimpleTask):
    """Compound Child representation for wrapping a Workflow TaskObject instance of
    TaskType Compound Child."""

    def __init__(self, command_source: WorkflowWrapper, task: str) -> None:
        """Initialize CompoundChild.

        Parameters
        ----------
        command_source : WorkflowWrapper
            The set of workflow commands.
        task : str
            The name of this task.
        """
        super().__init__(command_source, task)

    def python_name(self) -> str:
        """Get the Pythonic name of this task, as it is in the underlying application.

        Returns
        -------
        str
            The Pythonic name of this task.
        """
        pass


class CompositeTask(BaseTask):
    """Composite task representation for wrapping a Workflow TaskObject instance of
    TaskType Composite."""

    def __init__(self, command_source: WorkflowWrapper, task: str) -> None:
        """Initialize CompositeTask.

        Parameters
        ----------
        command_source : WorkflowWrapper
            The set of workflow commands.
        task : str
            The name of this task.
        """
        super().__init__(command_source, task)

    @property
    def CommandArguments(self) -> ReadOnlyObject:
        """Get the task's arguments in read-only form (deprecated).

        Returns
        -------
        ReadOnlyObject
            The task's arguments.
        """
        warnings.warn("CommandArguments", DeprecationWarning)
        return {}

    @property
    def _command_arguments(self) -> ReadOnlyObject:
        return {}

    @property
    def arguments(self) -> dict:
        """Get the task's arguments (empty for CompositeTask).

        Returns
        -------
        dict
            The task's arguments (empty).
        """
        return {}


class ConditionalTask(CommandTask):
    """Conditional task representation for wrapping a Workflow TaskObject instance of
    TaskType Conditional."""

    def __init__(self, command_source: WorkflowWrapper, task: str) -> None:
        """Initialize ConditionalTask.

        Parameters
        ----------
        command_source : WorkflowWrapper
            The set of workflow commands.
        task : str
            The name of this task.
        """
        super().__init__(command_source, task)

    def inactive_ordered_children(self) -> list:
        """Get the inactive ordered task list held by this task.

        Returns
        -------
        list
            Inactive ordered children.
        """
        inactive_task_list = self._task.InactiveTaskList()
        inactive_task_list = _convert_task_list_to_display_names(inactive_task_list)
        return [
            self._command_source._task_by_id(task_id) for task_id in inactive_task_list
        ]


class CompoundTask(CommandTask):
    """Compound task representation for wrapping a Workflow TaskObject instance of
    TaskType Compound."""

    def __init__(self, command_source: WorkflowWrapper, task: str) -> None:
        """Initialize CompoundTask.

        Parameters
        ----------
        command_source : WorkflowWrapper
            The set of workflow commands.
        task : str
            The name of this task.
        """
        super().__init__(command_source, task)

    def add_child(self, state: Optional[dict] = None) -> None:
        """Add a child to this CompoundTask.

        Parameters
        ----------
        state : Optional[dict]
            Optional state.
        """
        state = state or {}
        state.update({"AddChild": "yes"})
        self._task.Arguments.set_state(state)

    def add_child_and_update(self, state=None):
        """Add a child to this CompoundTask and update.

        Parameters
        ----------
        state : Optional[dict]
            Optional state.
        """
        self.add_child(state)
        self._task.AddChildAndUpdate()
        return self.last_child()

    def last_child(self) -> BaseTask:
        """Get the last child of this CompoundTask.

        Returns
        -------
        BaseTask
            the last child of this CompoundTask
        """
        children = self.ordered_children()
        if children:
            return children[-1]

    def compound_child(self, name: str):
        """Get the compound child task of this CompoundTask by name.

        Parameters
        ----------
        name : str
            name

        Returns
        -------
        BaseTask
            the named child of this CompoundTask
        """
        try:
            return next(filter(lambda t: t.name() == name, self.ordered_children()))
        except StopIteration:
            pass


def _makeTask(command_source, name: str) -> BaseTask:
    task = command_source._workflow.TaskObject[name]
    task_type = task.TaskType()
    kinds = {
        "Simple": SimpleTask,
        "Compound Child": CompoundChild,
        "Compound": CompoundTask,
        "Composite": CompositeTask,
        "Conditional": ConditionalTask,
    }
    kind = kinds[task.TaskType()]
    if not kind:
        message = (
            "Unhandled empty workflow task type."
            if not task.TaskType()
            else f"Unhandled workflow task type, {task.TaskType()}."
        )
        raise RuntimeError(message)
    return kind(command_source, task)


class WorkflowWrapper:
    """Wrap a Workflow object, adding methods to discover more about the relationships
    between TaskObjects.

    Methods
    -------
    task(name)
    ordered_children()
    __getattr__(attr)
    __dir__()
    __call__()
    """

    def __init__(self, workflow: PyMenuGeneric, command_source: PyMenuGeneric) -> None:
        """Initialize WorkflowWrapper.

        Parameters
        ----------
        workflow : PyMenuGeneric
            The workflow object.
        command_source : PyMenuGeneric
            The application root for commanding.
        """
        self._workflow = workflow
        self._command_source = command_source
        self._python_task_names = []
        self._lock = threading.RLock()
        self._refreshing = False
        self._refresh_count = 0
        self._ordered_children = []
        self._task_list = []
        self._getattr_recurse_depth = 0
        self._main_thread_ident = None
        self._task_objects = {}

    def task(self, name: str) -> BaseTask:
        """Get a TaskObject by name, in a BaseTask wrapper. The wrapper adds extra
        functionality.

        Parameters
        ----------
        name : str
            Task name - the display name, not the internal ID.

        Returns
        -------
        BaseTask
            wrapped task object.
        """
        return _makeTask(self, name)

    @property
    def TaskObject(self) -> TaskContainer:
        # missing from dir
        """Get a TaskObject container wrapper that 'holds' the underlying TaskObjects.

        The wrapper adds extra functionality.
        """
        return TaskContainer(self)

    def ordered_children(self, recompute=True) -> list:
        """Get the ordered task list held by the workflow. Sorting is in terms of the
        workflow order and only includes the top-level tasks, while other tasks can be
        obtained by calling ordered_children() on a parent task. Consider the following
        workflow.

        Given the workflow::

            Workflow
            ├── A
            ├── B
            │   ├── C
            │   └── D
            └── E

        The ordered children of the workflow are A, B, E, while B has ordered children C
        and D.
        """
        if recompute:
            workflow_state, task_list = self._workflow_and_task_list_state()

            def task_by_id(mappings):
                def _task_by_id(task_id):
                    if task_id in mappings:
                        return mappings[task_id]
                    try:
                        return self._task_by_id_impl(task_id, workflow_state)
                    except Exception:
                        pass

                return _task_by_id

            if task_list != self._task_list:
                mappings = {
                    k: v for k, v in zip(self._task_list, self._ordered_children)
                }
                self._ordered_children = list(
                    filter(None, map(task_by_id(mappings), task_list))
                )
                self._task_list = task_list
        return self._ordered_children

    def child_task_python_names(self) -> List[str]:
        """Get the Pythonic names of the child tasks.

        Returns
        -------
        List[str]
            Pythonic names of the child tasks.
        """
        with self._lock:
            return self._python_task_names

    def inactive_ordered_children(self) -> list:
        """Get the inactive ordered task list held by this task.

        Returns
        -------
        list
            Inactive ordered children.
        """
        return []

    def __getattr__(self, attr):
        """Delegate attribute lookup to the wrapped workflow object.

        Parameters
        ----------
        attr : str
            An attribute not defined in WorkflowWrapper
        """
        obj = self._attr_from_wrapped_workflow(
            attr
        )  # or self._task_with_cmd_matching_help_string(attr)
        if obj:
            return obj
        return self._task_objects.get(attr, None)

    def __dir__(self):
        """Override the behaviour of dir to include attributes in WorkflowWrapper and
        the underlying workflow."""
        return sorted(
            set(list(self.__dict__.keys()) + dir(type(self)) + dir(self._workflow))
        )

    def __call__(self):
        """Delegate calls to the underlying workflow."""
        return self._workflow()

    def _workflow_state(self):
        return self._workflow()

    def _workflow_and_task_list_state(self) -> Tuple[dict, dict]:
        workflow_state = self._workflow_state()
        prefix = "TaskObject:"
        task_list = [
            x.removeprefix(prefix)
            for x in workflow_state.keys()
            if x.startswith(prefix)
        ]
        return workflow_state, task_list

    def _task_by_id_impl(self, task_id, workflow_state):
        task_key = "TaskObject:" + task_id
        task_state = workflow_state[task_key]
        return self.task(task_state["_name_"])

    def _task_by_id(self, task_id):
        workflow_state = self._workflow_state()
        return self._task_by_id_impl(task_id, workflow_state)

    def _attr_from_wrapped_workflow(self, attr):
        try:
            result = getattr(self._workflow, attr)
            if result:
                return result
        except AttributeError:
            pass

    def _new_workflow(self, name: str, dynamic_interface: bool):
        self._workflow.InitializeWorkflow(WorkflowType=name)
        self._initialize_methods(dynamic_interface=dynamic_interface)

    def _initialize_methods(self, dynamic_interface: bool):
        _init_task_accessors(self)
        if dynamic_interface:
            self._main_thread_ident = threading.get_ident()
            logger.debug(f"setting main thread to {self._main_thread_ident}")

            def refresh_after_sleep(_):
                while self._refreshing:
                    logger.debug("Already _refreshing, ...")
                self._refreshing = True
                logger.debug("Call _refresh_task_accessors")
                _refresh_task_accessors(self)
                self._refresh_count += 1
                self._refreshing = False

            self.add_on_affected(refresh_after_sleep)


class ReadOnlyObject:
    """Removes set_state() to implement read-only behaviour."""

    _unwanted_attr = ["set_state", "setState"]

    def __init__(self, cmd):
        """Initialize this object."""
        self._cmd = cmd

    def is_read_only(self):
        """Get the read-only status of this object."""
        return True

    def __getattr__(self, attr):
        if attr in ReadOnlyObject._unwanted_attr:
            raise AttributeError("Command Arguments are read-only.")
        return getattr(self._cmd, attr)

    def __dir__(self):
        returned_list = sorted(
            set(list(self.__dict__.keys()) + dir(type(self)) + dir(self._cmd))
        )
        for attr in ReadOnlyObject._unwanted_attr:
            if attr in returned_list:
                returned_list.remove(attr)
        return returned_list

    def __call__(self):
        return self._cmd()



#
# This is an auto-generated file.  DO NOT EDIT!
#


from typing import Union, List, Tuple

class mesh:
    fluent_name = ...
    command_names = ...

    def display(self, object_name: str):
        """
        Display graphics object.
        
        Parameters
        ----------
            object_name : str
                'object_name' child.
        
        """

    def copy(self, from_name: str, new_name: str):
        """
        Copy graphics object.
        
        Parameters
        ----------
            from_name : str
                'from_name' child.
            new_name : str
                'new_name' child.
        
        """

    def add_to_graphics(self, object_name: str):
        """
        Add graphics object to existing graphics.
        
        Parameters
        ----------
            object_name : str
                'object_name' child.
        
        """

    def clear_history(self, object_name: str):
        """
        Clear object history.
        
        Parameters
        ----------
            object_name : str
                'object_name' child.
        
        """

    <class 'ansys.fluent.core.solver.flobject.child_object_type'> = ...



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .name_1 import name as name_cls
from .report_type import report_type as report_type_cls
from .old_props import old_props as old_props_cls
class icing_child(Group):
    """
    'child_object_type' of icing.
    """

    fluent_name = "child-object-type"

    child_names = \
        ['name', 'report_type', 'old_props']

    name: name_cls = name_cls
    """
    name child of icing_child.
    """
    report_type: report_type_cls = report_type_cls
    """
    report_type child of icing_child.
    """
    old_props: old_props_cls = old_props_cls
    """
    old_props child of icing_child.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .general import general as general_cls
from .models_1 import models as models_cls
from .materials import materials as materials_cls
from .cell_zone_conditions import cell_zone_conditions as cell_zone_conditions_cls
from .boundary_conditions import boundary_conditions as boundary_conditions_cls
from .mesh_interfaces import mesh_interfaces as mesh_interfaces_cls
from .reference_values import reference_values as reference_values_cls
from .reference_frames import reference_frames as reference_frames_cls
from .named_expressions import named_expressions as named_expressions_cls
from .turbo_models import turbo_models as turbo_models_cls
from .geometry_4 import geometry as geometry_cls
from .physics import physics as physics_cls
class setup(Group):
    """
    'setup' child.
    """

    fluent_name = "setup"

    child_names = \
        ['general', 'models', 'materials', 'cell_zone_conditions',
         'boundary_conditions', 'mesh_interfaces', 'reference_values',
         'reference_frames', 'named_expressions', 'turbo_models', 'geometry',
         'physics']

    general: general_cls = general_cls
    """
    general child of setup.
    """
    models: models_cls = models_cls
    """
    models child of setup.
    """
    materials: materials_cls = materials_cls
    """
    materials child of setup.
    """
    cell_zone_conditions: cell_zone_conditions_cls = cell_zone_conditions_cls
    """
    cell_zone_conditions child of setup.
    """
    boundary_conditions: boundary_conditions_cls = boundary_conditions_cls
    """
    boundary_conditions child of setup.
    """
    mesh_interfaces: mesh_interfaces_cls = mesh_interfaces_cls
    """
    mesh_interfaces child of setup.
    """
    reference_values: reference_values_cls = reference_values_cls
    """
    reference_values child of setup.
    """
    reference_frames: reference_frames_cls = reference_frames_cls
    """
    reference_frames child of setup.
    """
    named_expressions: named_expressions_cls = named_expressions_cls
    """
    named_expressions child of setup.
    """
    turbo_models: turbo_models_cls = turbo_models_cls
    """
    turbo_models child of setup.
    """
    geometry: geometry_cls = geometry_cls
    """
    geometry child of setup.
    """
    physics: physics_cls = physics_cls
    """
    physics child of setup.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .merge_small_regions import merge_small_regions as merge_small_regions_cls
from .max_merge_iterations import max_merge_iterations as max_merge_iterations_cls
class merge(Group):
    """
    Set partition merging optimization.
    """

    fluent_name = "merge"

    child_names = \
        ['merge_small_regions', 'max_merge_iterations']

    merge_small_regions: merge_small_regions_cls = merge_small_regions_cls
    """
    merge_small_regions child of merge.
    """
    max_merge_iterations: max_merge_iterations_cls = max_merge_iterations_cls
    """
    max_merge_iterations child of merge.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .thermal_bc import thermal_bc as thermal_bc_cls
from .temperature_1 import temperature as temperature_cls
from .q import q as q_cls
class phase_child(Group):
    """
    'child_object_type' of phase.
    """

    fluent_name = "child-object-type"

    child_names = \
        ['thermal_bc', 'temperature', 'q']

    thermal_bc: thermal_bc_cls = thermal_bc_cls
    """
    thermal_bc child of phase_child.
    """
    temperature: temperature_cls = temperature_cls
    """
    temperature child of phase_child.
    """
    q: q_cls = q_cls
    """
    q child of phase_child.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .option import option as option_cls
from .hexahedron import hexahedron as hexahedron_cls
from .sphere import sphere as sphere_cls
from .cylinder import cylinder as cylinder_cls
from .boundary_3 import boundary as boundary_cls
from .limiters import limiters as limiters_cls
from .field_value import field_value as field_value_cls
from .residual_1 import residual as residual_cls
from .volume_1 import volume as volume_cls
from .yplus_star import yplus_star as yplus_star_cls
from .yplus_ystar import yplus_ystar as yplus_ystar_cls
class type(Group):
    """
    'type' child.
    """

    fluent_name = "type"

    child_names = \
        ['option', 'hexahedron', 'sphere', 'cylinder', 'boundary', 'limiters',
         'field_value', 'residual', 'volume', 'yplus_star', 'yplus_ystar']

    option: option_cls = option_cls
    """
    option child of type.
    """
    hexahedron: hexahedron_cls = hexahedron_cls
    """
    hexahedron child of type.
    """
    sphere: sphere_cls = sphere_cls
    """
    sphere child of type.
    """
    cylinder: cylinder_cls = cylinder_cls
    """
    cylinder child of type.
    """
    boundary: boundary_cls = boundary_cls
    """
    boundary child of type.
    """
    limiters: limiters_cls = limiters_cls
    """
    limiters child of type.
    """
    field_value: field_value_cls = field_value_cls
    """
    field_value child of type.
    """
    residual: residual_cls = residual_cls
    """
    residual child of type.
    """
    volume: volume_cls = volume_cls
    """
    volume child of type.
    """
    yplus_star: yplus_star_cls = yplus_star_cls
    """
    yplus_star child of type.
    """
    yplus_ystar: yplus_ystar_cls = yplus_ystar_cls
    """
    yplus_ystar child of type.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#


from typing import Union, List, Tuple

class volumes_child:
    fluent_name = ...
    child_names = ...
    name = ...
    home_options = ...
    transparency_options = ...
    isovalue_options = ...
    clip_box_options = ...
    clip_sphere_options = ...
    iso_values_colors_list = ...
    local_values_range_list = ...
    opacities_range_list = ...
    transfer_functions_list = ...
    clip_bound_values_list = ...
    volume_data_list = ...
    custom_update_attribute_list = ...
    compute_node_count = ...
    command_names = ...

    def display(self, ):
        """
        'display' command.
        """




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .option import option as option_cls
from .constant import constant as constant_cls
from .boussinesq import boussinesq as boussinesq_cls
from .coefficients import coefficients as coefficients_cls
from .number_of_coefficients import number_of_coefficients as number_of_coefficients_cls
from .piecewise_polynomial import piecewise_polynomial as piecewise_polynomial_cls
from .nasa_9_piecewise_polynomial import nasa_9_piecewise_polynomial as nasa_9_piecewise_polynomial_cls
from .piecewise_linear import piecewise_linear as piecewise_linear_cls
from .anisotropic import anisotropic as anisotropic_cls
from .orthotropic import orthotropic as orthotropic_cls
from .var_class import var_class as var_class_cls
class molecular_weight(Group):
    """
    'molecular_weight' child.
    """

    fluent_name = "molecular-weight"

    child_names = \
        ['option', 'constant', 'boussinesq', 'coefficients',
         'number_of_coefficients', 'piecewise_polynomial',
         'nasa_9_piecewise_polynomial', 'piecewise_linear', 'anisotropic',
         'orthotropic', 'var_class']

    option: option_cls = option_cls
    """
    option child of molecular_weight.
    """
    constant: constant_cls = constant_cls
    """
    constant child of molecular_weight.
    """
    boussinesq: boussinesq_cls = boussinesq_cls
    """
    boussinesq child of molecular_weight.
    """
    coefficients: coefficients_cls = coefficients_cls
    """
    coefficients child of molecular_weight.
    """
    number_of_coefficients: number_of_coefficients_cls = number_of_coefficients_cls
    """
    number_of_coefficients child of molecular_weight.
    """
    piecewise_polynomial: piecewise_polynomial_cls = piecewise_polynomial_cls
    """
    piecewise_polynomial child of molecular_weight.
    """
    nasa_9_piecewise_polynomial: nasa_9_piecewise_polynomial_cls = nasa_9_piecewise_polynomial_cls
    """
    nasa_9_piecewise_polynomial child of molecular_weight.
    """
    piecewise_linear: piecewise_linear_cls = piecewise_linear_cls
    """
    piecewise_linear child of molecular_weight.
    """
    anisotropic: anisotropic_cls = anisotropic_cls
    """
    anisotropic child of molecular_weight.
    """
    orthotropic: orthotropic_cls = orthotropic_cls
    """
    orthotropic child of molecular_weight.
    """
    var_class: var_class_cls = var_class_cls
    """
    var_class child of molecular_weight.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#


from typing import Union, List, Tuple

class sample_trajectories:
    fluent_name = ...
    child_names = ...
    user_defined_functions = ...
    sort_sample_files = ...
    command_names = ...

    def sample(self, injections: List[str], boundaries: List[str], lines: List[str], planes: List[str], op_udf: str, append_sample: bool, accumulate_rates: bool):
        """
        'sample' command.
        
        Parameters
        ----------
            injections : typing.List[str]
                'injections' child.
            boundaries : typing.List[str]
                'boundaries' child.
            lines : typing.List[str]
                'lines' child.
            planes : typing.List[str]
                'planes' child.
            op_udf : str
                'op_udf' child.
            append_sample : bool
                'append_sample' child.
            accumulate_rates : bool
                'accumulate_rates' child.
        
        """




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .open_channel import open_channel as open_channel_cls
from .inlet_number import inlet_number as inlet_number_cls
from .phase_spec_1 import phase_spec as phase_spec_cls
from .flow_spec import flow_spec as flow_spec_cls
from .free_surface_level import free_surface_level as free_surface_level_cls
from .ht_bottom import ht_bottom as ht_bottom_cls
from .ht_total import ht_total as ht_total_cls
from .vmag import vmag as vmag_cls
from .den_spec import den_spec as den_spec_cls
from .granular_temperature_1 import granular_temperature as granular_temperature_cls
from .interfacial_area_concentration_1 import interfacial_area_concentration as interfacial_area_concentration_cls
from .level_set_function_flux_1 import level_set_function_flux as level_set_function_flux_cls
from .volume_fraction_1 import volume_fraction as volume_fraction_cls
from .pb_disc_bc import pb_disc_bc as pb_disc_bc_cls
from .pb_disc import pb_disc as pb_disc_cls
from .pb_qmom_bc import pb_qmom_bc as pb_qmom_bc_cls
from .pb_qmom import pb_qmom as pb_qmom_cls
from .pb_qbmm_bc import pb_qbmm_bc as pb_qbmm_bc_cls
from .pb_qbmm import pb_qbmm as pb_qbmm_cls
from .pb_smm_bc import pb_smm_bc as pb_smm_bc_cls
from .pb_smm import pb_smm as pb_smm_cls
from .pb_dqmom_bc import pb_dqmom_bc as pb_dqmom_bc_cls
from .pb_dqmom import pb_dqmom as pb_dqmom_cls
from .wsf_1 import wsf as wsf_cls
from .wsb_1 import wsb as wsb_cls
from .wsn_1 import wsn as wsn_cls
class multiphase(Group):
    """
    Help not available.
    """

    fluent_name = "multiphase"

    child_names = \
        ['open_channel', 'inlet_number', 'phase_spec', 'flow_spec',
         'free_surface_level', 'ht_bottom', 'ht_total', 'vmag', 'den_spec',
         'granular_temperature', 'interfacial_area_concentration',
         'level_set_function_flux', 'volume_fraction', 'pb_disc_bc',
         'pb_disc', 'pb_qmom_bc', 'pb_qmom', 'pb_qbmm_bc', 'pb_qbmm',
         'pb_smm_bc', 'pb_smm', 'pb_dqmom_bc', 'pb_dqmom', 'wsf', 'wsb',
         'wsn']

    open_channel: open_channel_cls = open_channel_cls
    """
    open_channel child of multiphase.
    """
    inlet_number: inlet_number_cls = inlet_number_cls
    """
    inlet_number child of multiphase.
    """
    phase_spec: phase_spec_cls = phase_spec_cls
    """
    phase_spec child of multiphase.
    """
    flow_spec: flow_spec_cls = flow_spec_cls
    """
    flow_spec child of multiphase.
    """
    free_surface_level: free_surface_level_cls = free_surface_level_cls
    """
    free_surface_level child of multiphase.
    """
    ht_bottom: ht_bottom_cls = ht_bottom_cls
    """
    ht_bottom child of multiphase.
    """
    ht_total: ht_total_cls = ht_total_cls
    """
    ht_total child of multiphase.
    """
    vmag: vmag_cls = vmag_cls
    """
    vmag child of multiphase.
    """
    den_spec: den_spec_cls = den_spec_cls
    """
    den_spec child of multiphase.
    """
    granular_temperature: granular_temperature_cls = granular_temperature_cls
    """
    granular_temperature child of multiphase.
    """
    interfacial_area_concentration: interfacial_area_concentration_cls = interfacial_area_concentration_cls
    """
    interfacial_area_concentration child of multiphase.
    """
    level_set_function_flux: level_set_function_flux_cls = level_set_function_flux_cls
    """
    level_set_function_flux child of multiphase.
    """
    volume_fraction: volume_fraction_cls = volume_fraction_cls
    """
    volume_fraction child of multiphase.
    """
    pb_disc_bc: pb_disc_bc_cls = pb_disc_bc_cls
    """
    pb_disc_bc child of multiphase.
    """
    pb_disc: pb_disc_cls = pb_disc_cls
    """
    pb_disc child of multiphase.
    """
    pb_qmom_bc: pb_qmom_bc_cls = pb_qmom_bc_cls
    """
    pb_qmom_bc child of multiphase.
    """
    pb_qmom: pb_qmom_cls = pb_qmom_cls
    """
    pb_qmom child of multiphase.
    """
    pb_qbmm_bc: pb_qbmm_bc_cls = pb_qbmm_bc_cls
    """
    pb_qbmm_bc child of multiphase.
    """
    pb_qbmm: pb_qbmm_cls = pb_qbmm_cls
    """
    pb_qbmm child of multiphase.
    """
    pb_smm_bc: pb_smm_bc_cls = pb_smm_bc_cls
    """
    pb_smm_bc child of multiphase.
    """
    pb_smm: pb_smm_cls = pb_smm_cls
    """
    pb_smm child of multiphase.
    """
    pb_dqmom_bc: pb_dqmom_bc_cls = pb_dqmom_bc_cls
    """
    pb_dqmom_bc child of multiphase.
    """
    pb_dqmom: pb_dqmom_cls = pb_dqmom_cls
    """
    pb_dqmom child of multiphase.
    """
    wsf: wsf_cls = wsf_cls
    """
    wsf child of multiphase.
    """
    wsb: wsb_cls = wsb_cls
    """
    wsb child of multiphase.
    """
    wsn: wsn_cls = wsn_cls
    """
    wsn child of multiphase.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .use_inherent_material_color_1 import use_inherent_material_color as use_inherent_material_color_cls
from .reset_2 import reset as reset_cls
from .list_surfaces_by_color import list_surfaces_by_color as list_surfaces_by_color_cls
from .list_surfaces_by_material import list_surfaces_by_material as list_surfaces_by_material_cls
from .surfaces_7 import surfaces as surfaces_cls
class by_surface(Group):
    """
    'by_surface' child.
    """

    fluent_name = "by-surface"

    child_names = \
        ['use_inherent_material_color']

    use_inherent_material_color: use_inherent_material_color_cls = use_inherent_material_color_cls
    """
    use_inherent_material_color child of by_surface.
    """
    command_names = \
        ['reset', 'list_surfaces_by_color', 'list_surfaces_by_material',
         'surfaces']

    reset: reset_cls = reset_cls
    """
    reset command of by_surface.
    """
    list_surfaces_by_color: list_surfaces_by_color_cls = list_surfaces_by_color_cls
    """
    list_surfaces_by_color command of by_surface.
    """
    list_surfaces_by_material: list_surfaces_by_material_cls = list_surfaces_by_material_cls
    """
    list_surfaces_by_material command of by_surface.
    """
    surfaces: surfaces_cls = surfaces_cls
    """
    surfaces command of by_surface.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .phase_1 import phase as phase_cls
from .name_1 import name as name_cls
from .material import material as material_cls
from .sources import sources as sources_cls
from .source_terms_1 import source_terms as source_terms_cls
from .fixed import fixed as fixed_cls
from .cylindrical_fixed_var import cylindrical_fixed_var as cylindrical_fixed_var_cls
from .fixes import fixes as fixes_cls
from .motion_spec import motion_spec as motion_spec_cls
from .relative_to_thread import relative_to_thread as relative_to_thread_cls
from .omega import omega as omega_cls
from .axis_origin_1 import axis_origin as axis_origin_cls
from .axis_direction_1 import axis_direction as axis_direction_cls
from .udf_zmotion_name import udf_zmotion_name as udf_zmotion_name_cls
from .mrf_motion import mrf_motion as mrf_motion_cls
from .mrf_relative_to_thread import mrf_relative_to_thread as mrf_relative_to_thread_cls
from .mrf_omega import mrf_omega as mrf_omega_cls
from .reference_frame_velocity import reference_frame_velocity as reference_frame_velocity_cls
from .reference_frame_axis_origin import reference_frame_axis_origin as reference_frame_axis_origin_cls
from .reference_frame_axis_direction import reference_frame_axis_direction as reference_frame_axis_direction_cls
from .mrf_udf_zmotion_name import mrf_udf_zmotion_name as mrf_udf_zmotion_name_cls
from .mgrid_enable_transient import mgrid_enable_transient as mgrid_enable_transient_cls
from .mgrid_motion import mgrid_motion as mgrid_motion_cls
from .mgrid_relative_to_thread import mgrid_relative_to_thread as mgrid_relative_to_thread_cls
from .mgrid_omega import mgrid_omega as mgrid_omega_cls
from .moving_mesh_velocity import moving_mesh_velocity as moving_mesh_velocity_cls
from .moving_mesh_axis_origin import moving_mesh_axis_origin as moving_mesh_axis_origin_cls
from .moving_mesh_axis_direction import moving_mesh_axis_direction as moving_mesh_axis_direction_cls
from .mgrid_udf_zmotion_name import mgrid_udf_zmotion_name as mgrid_udf_zmotion_name_cls
from .solid_motion import solid_motion as solid_motion_cls
from .solid_relative_to_thread import solid_relative_to_thread as solid_relative_to_thread_cls
from .solid_omega import solid_omega as solid_omega_cls
from .solid_motion_velocity import solid_motion_velocity as solid_motion_velocity_cls
from .solid_motion_axis_origin import solid_motion_axis_origin as solid_motion_axis_origin_cls
from .solid_motion_axis_direction import solid_motion_axis_direction as solid_motion_axis_direction_cls
from .solid_udf_zmotion_name import solid_udf_zmotion_name as solid_udf_zmotion_name_cls
from .radiating import radiating as radiating_cls
from .les_embedded import les_embedded as les_embedded_cls
from .contact_property import contact_property as contact_property_cls
from .active_wetsteam_zone import active_wetsteam_zone as active_wetsteam_zone_cls
from .vapor_phase_realgas import vapor_phase_realgas as vapor_phase_realgas_cls
from .cursys import cursys as cursys_cls
from .cursys_name import cursys_name as cursys_name_cls
from .pcb_model import pcb_model as pcb_model_cls
from .pcb_zone_info import pcb_zone_info as pcb_zone_info_cls
class solid_child(Group):
    """
    'child_object_type' of solid.
    """

    fluent_name = "child-object-type"

    child_names = \
        ['phase', 'name', 'material', 'sources', 'source_terms', 'fixed',
         'cylindrical_fixed_var', 'fixes', 'motion_spec',
         'relative_to_thread', 'omega', 'axis_origin', 'axis_direction',
         'udf_zmotion_name', 'mrf_motion', 'mrf_relative_to_thread',
         'mrf_omega', 'reference_frame_velocity',
         'reference_frame_axis_origin', 'reference_frame_axis_direction',
         'mrf_udf_zmotion_name', 'mgrid_enable_transient', 'mgrid_motion',
         'mgrid_relative_to_thread', 'mgrid_omega', 'moving_mesh_velocity',
         'moving_mesh_axis_origin', 'moving_mesh_axis_direction',
         'mgrid_udf_zmotion_name', 'solid_motion', 'solid_relative_to_thread',
         'solid_omega', 'solid_motion_velocity', 'solid_motion_axis_origin',
         'solid_motion_axis_direction', 'solid_udf_zmotion_name', 'radiating',
         'les_embedded', 'contact_property', 'active_wetsteam_zone',
         'vapor_phase_realgas', 'cursys', 'cursys_name', 'pcb_model',
         'pcb_zone_info']

    phase: phase_cls = phase_cls
    """
    phase child of solid_child.
    """
    name: name_cls = name_cls
    """
    name child of solid_child.
    """
    material: material_cls = material_cls
    """
    material child of solid_child.
    """
    sources: sources_cls = sources_cls
    """
    sources child of solid_child.
    """
    source_terms: source_terms_cls = source_terms_cls
    """
    source_terms child of solid_child.
    """
    fixed: fixed_cls = fixed_cls
    """
    fixed child of solid_child.
    """
    cylindrical_fixed_var: cylindrical_fixed_var_cls = cylindrical_fixed_var_cls
    """
    cylindrical_fixed_var child of solid_child.
    """
    fixes: fixes_cls = fixes_cls
    """
    fixes child of solid_child.
    """
    motion_spec: motion_spec_cls = motion_spec_cls
    """
    motion_spec child of solid_child.
    """
    relative_to_thread: relative_to_thread_cls = relative_to_thread_cls
    """
    relative_to_thread child of solid_child.
    """
    omega: omega_cls = omega_cls
    """
    omega child of solid_child.
    """
    axis_origin: axis_origin_cls = axis_origin_cls
    """
    axis_origin child of solid_child.
    """
    axis_direction: axis_direction_cls = axis_direction_cls
    """
    axis_direction child of solid_child.
    """
    udf_zmotion_name: udf_zmotion_name_cls = udf_zmotion_name_cls
    """
    udf_zmotion_name child of solid_child.
    """
    mrf_motion: mrf_motion_cls = mrf_motion_cls
    """
    mrf_motion child of solid_child.
    """
    mrf_relative_to_thread: mrf_relative_to_thread_cls = mrf_relative_to_thread_cls
    """
    mrf_relative_to_thread child of solid_child.
    """
    mrf_omega: mrf_omega_cls = mrf_omega_cls
    """
    mrf_omega child of solid_child.
    """
    reference_frame_velocity: reference_frame_velocity_cls = reference_frame_velocity_cls
    """
    reference_frame_velocity child of solid_child.
    """
    reference_frame_axis_origin: reference_frame_axis_origin_cls = reference_frame_axis_origin_cls
    """
    reference_frame_axis_origin child of solid_child.
    """
    reference_frame_axis_direction: reference_frame_axis_direction_cls = reference_frame_axis_direction_cls
    """
    reference_frame_axis_direction child of solid_child.
    """
    mrf_udf_zmotion_name: mrf_udf_zmotion_name_cls = mrf_udf_zmotion_name_cls
    """
    mrf_udf_zmotion_name child of solid_child.
    """
    mgrid_enable_transient: mgrid_enable_transient_cls = mgrid_enable_transient_cls
    """
    mgrid_enable_transient child of solid_child.
    """
    mgrid_motion: mgrid_motion_cls = mgrid_motion_cls
    """
    mgrid_motion child of solid_child.
    """
    mgrid_relative_to_thread: mgrid_relative_to_thread_cls = mgrid_relative_to_thread_cls
    """
    mgrid_relative_to_thread child of solid_child.
    """
    mgrid_omega: mgrid_omega_cls = mgrid_omega_cls
    """
    mgrid_omega child of solid_child.
    """
    moving_mesh_velocity: moving_mesh_velocity_cls = moving_mesh_velocity_cls
    """
    moving_mesh_velocity child of solid_child.
    """
    moving_mesh_axis_origin: moving_mesh_axis_origin_cls = moving_mesh_axis_origin_cls
    """
    moving_mesh_axis_origin child of solid_child.
    """
    moving_mesh_axis_direction: moving_mesh_axis_direction_cls = moving_mesh_axis_direction_cls
    """
    moving_mesh_axis_direction child of solid_child.
    """
    mgrid_udf_zmotion_name: mgrid_udf_zmotion_name_cls = mgrid_udf_zmotion_name_cls
    """
    mgrid_udf_zmotion_name child of solid_child.
    """
    solid_motion: solid_motion_cls = solid_motion_cls
    """
    solid_motion child of solid_child.
    """
    solid_relative_to_thread: solid_relative_to_thread_cls = solid_relative_to_thread_cls
    """
    solid_relative_to_thread child of solid_child.
    """
    solid_omega: solid_omega_cls = solid_omega_cls
    """
    solid_omega child of solid_child.
    """
    solid_motion_velocity: solid_motion_velocity_cls = solid_motion_velocity_cls
    """
    solid_motion_velocity child of solid_child.
    """
    solid_motion_axis_origin: solid_motion_axis_origin_cls = solid_motion_axis_origin_cls
    """
    solid_motion_axis_origin child of solid_child.
    """
    solid_motion_axis_direction: solid_motion_axis_direction_cls = solid_motion_axis_direction_cls
    """
    solid_motion_axis_direction child of solid_child.
    """
    solid_udf_zmotion_name: solid_udf_zmotion_name_cls = solid_udf_zmotion_name_cls
    """
    solid_udf_zmotion_name child of solid_child.
    """
    radiating: radiating_cls = radiating_cls
    """
    radiating child of solid_child.
    """
    les_embedded: les_embedded_cls = les_embedded_cls
    """
    les_embedded child of solid_child.
    """
    contact_property: contact_property_cls = contact_property_cls
    """
    contact_property child of solid_child.
    """
    active_wetsteam_zone: active_wetsteam_zone_cls = active_wetsteam_zone_cls
    """
    active_wetsteam_zone child of solid_child.
    """
    vapor_phase_realgas: vapor_phase_realgas_cls = vapor_phase_realgas_cls
    """
    vapor_phase_realgas child of solid_child.
    """
    cursys: cursys_cls = cursys_cls
    """
    cursys child of solid_child.
    """
    cursys_name: cursys_name_cls = cursys_name_cls
    """
    cursys_name child of solid_child.
    """
    pcb_model: pcb_model_cls = pcb_model_cls
    """
    pcb_model child of solid_child.
    """
    pcb_zone_info: pcb_zone_info_cls = pcb_zone_info_cls
    """
    pcb_zone_info child of solid_child.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .file_name_1 import file_name as file_name_cls
from .view_list import view_list as view_list_cls
class write_views(Command):
    """
    Write selected views to a view file.
    
    Parameters
    ----------
        file_name : str
            'file_name' child.
        view_list : typing.List[str]
            'view_list' child.
    
    """

    fluent_name = "write-views"

    argument_names = \
        ['file_name', 'view_list']

    file_name: file_name_cls = file_name_cls
    """
    file_name argument of write_views.
    """
    view_list: view_list_cls = view_list_cls
    """
    view_list argument of write_views.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .kw_low_re_correction import kw_low_re_correction as kw_low_re_correction_cls
from .kw_shear_correction import kw_shear_correction as kw_shear_correction_cls
class k_omega_options(Group):
    """
    'k_omega_options' child.
    """

    fluent_name = "k-omega-options"

    child_names = \
        ['kw_low_re_correction', 'kw_shear_correction']

    kw_low_re_correction: kw_low_re_correction_cls = kw_low_re_correction_cls
    """
    kw_low_re_correction child of k_omega_options.
    """
    kw_shear_correction: kw_shear_correction_cls = kw_shear_correction_cls
    """
    kw_shear_correction child of k_omega_options.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#


from typing import Union, List, Tuple

class lights:
    fluent_name = ...
    child_names = ...
    ambient_color = ...
    headlight_setting = ...
    lights_on = ...
    lighting_interpolation = ...
    command_names = ...

    def set_light(self, light_number: int, light_on: bool, rgb_vector: List[Union[float, str]], use_view_factor: bool, change_light_direction: bool, direction_vector: List[Union[float, str]]):
        """
        'set_light' command.
        
        Parameters
        ----------
            light_number : int
                'light_number' child.
            light_on : bool
                'light_on' child.
            rgb_vector : typing.List[real]
                'rgb_vector' child.
            use_view_factor : bool
                'use_view_factor' child.
            change_light_direction : bool
                'change_light_direction' child.
            direction_vector : typing.List[real]
                'direction_vector' child.
        
        """




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .remove_dataset import remove_dataset as remove_dataset_cls
from .list_datasets import list_datasets as list_datasets_cls
class data_sampling_options(Group):
    """
    Data sampling options for statistics.
    """

    fluent_name = "data-sampling-options"

    command_names = \
        ['remove_dataset', 'list_datasets']

    remove_dataset: remove_dataset_cls = remove_dataset_cls
    """
    remove_dataset command of data_sampling_options.
    """
    list_datasets: list_datasets_cls = list_datasets_cls
    """
    list_datasets command of data_sampling_options.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .name import name as name_cls
from .surface import surface as surface_cls
from .zones_4 import zones as zones_cls
from .display_3 import display as display_cls
class imprint_surface_child(Group):
    """
    'child_object_type' of imprint_surface.
    """

    fluent_name = "child-object-type"

    child_names = \
        ['name', 'surface', 'zones']

    name: name_cls = name_cls
    """
    name child of imprint_surface_child.
    """
    surface: surface_cls = surface_cls
    """
    surface child of imprint_surface_child.
    """
    zones: zones_cls = zones_cls
    """
    zones child of imprint_surface_child.
    """
    command_names = \
        ['display']

    display: display_cls = display_cls
    """
    display command of imprint_surface_child.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .compute_statistics import compute_statistics as compute_statistics_cls
from .statistics_level import statistics_level as statistics_level_cls
class miscellaneous(Group):
    """
    Enter flexible numeris menu for solution statics options.
    """

    fluent_name = "miscellaneous"

    child_names = \
        ['compute_statistics', 'statistics_level']

    compute_statistics: compute_statistics_cls = compute_statistics_cls
    """
    compute_statistics child of miscellaneous.
    """
    statistics_level: statistics_level_cls = statistics_level_cls
    """
    statistics_level child of miscellaneous.
    """



from pathlib import Path

import pytest

from ansys.fluent.core import examples
from ansys.fluent.core.exceptions import SurfaceSpecificationError
from ansys.fluent.core.file_session import (
    FileSession,
    InvalidFieldName,
    InvalidMultiPhaseFieldName,
)
from ansys.fluent.core.services.field_data import SurfaceDataType


def round_off_list_elements(input_list):
    for index, value in enumerate(input_list):
        input_list[index] = round(value, 6)

    return input_list


def test_field_info_data_multi_phase():
    case_file_name = examples.download_file(
        "mixing_elbow_mul_ph.cas.h5",
        "pyfluent/file_session",
        return_without_path=False,
    )
    data_file_name = examples.download_file(
        "mixing_elbow_mul_ph.dat.h5",
        "pyfluent/file_session",
        return_without_path=False,
    )
    file_session = FileSession()
    assert Path(case_file_name).exists()
    assert Path(data_file_name).exists()
    file_session.read_case(case_file_name)
    file_session.read_data(data_file_name)

    sv_density = file_session.field_data.get_scalar_field_data(
        "phase-2:SV_DENSITY", [33]
    )
    assert sv_density[33].size == 268
    assert sv_density[33][130].scalar_data == 1.225
    assert (
        round(
            file_session.field_data.get_scalar_field_data(
                "phase-2:SV_WALL_YPLUS", [33]
            )[33][130].scalar_data,
            5,
        )
        == 0.00103
    )
    vector_data = file_session.field_data.get_vector_field_data
    assert vector_data("phase-2:velocity", surface_ids=[33])[33].size == 268
    assert vector_data("phase-1:velocity", surface_ids=[34])[34].size == 2168


def test_field_info_data_single_phase():
    case_file_name = examples.download_file(
        "elbow1.cas.h5", "pyfluent/file_session", return_without_path=False
    )
    data_file_name = examples.download_file(
        "elbow1.dat.h5", "pyfluent/file_session", return_without_path=False
    )
    file_session = FileSession()
    file_session.read_case(case_file_name)
    file_session.read_data(data_file_name)

    assert round_off_list_elements(
        file_session.field_info.get_scalar_field_range("SV_P")
    ) == [-339.203452, 339.417934]
    assert len(file_session.field_info.get_scalar_fields_info()) == 29
    assert list(file_session.field_info.get_surfaces_info().keys()) == [
        "wall",
        "symmetry",
        "pressure-outlet-7",
        "velocity-inlet-6",
        "velocity-inlet-5",
        "default-interior",
    ]
    sv_t_wall = file_session.field_data.get_scalar_field_data(
        "SV_T", surface_name="wall"
    )
    assert sv_t_wall.size == 3630
    assert round(sv_t_wall[1800].scalar_data, 4) == 313.15

    surface_data = file_session.field_data.get_surface_data
    surface_data_wall = surface_data(
        data_type=SurfaceDataType.Vertices, surface_name="wall"
    )
    assert surface_data_wall.size == 3810
    assert round(surface_data_wall[1500].x, 5) == 0.12406
    assert round(surface_data_wall[1500].y, 5) == 0.09525
    assert round(surface_data_wall[1500].z, 5) == 0.04216

    surface_data_symmetry = surface_data(
        data_type=SurfaceDataType.FacesConnectivity, surface_name="symmetry"
    )
    assert surface_data_symmetry.size == 2018
    assert surface_data_symmetry[1000].node_count == 4
    assert list(surface_data_symmetry[1000].node_indices) == [1259, 1260, 1227, 1226]

    vector_data = file_session.field_data.get_vector_field_data
    assert vector_data("velocity", surface_name="wall").size == 3630

    vector_data_symmetry = vector_data("velocity", surface_name="symmetry")
    assert vector_data_symmetry.size == 2018
    assert round(vector_data_symmetry[1009].x, 5) == 0.0023
    assert round(vector_data_symmetry[1009].y, 5) == 1.22311


def test_data_reader_single_phase():
    case_file_name = examples.download_file(
        "elbow1.cas.h5", "pyfluent/file_session", return_without_path=False
    )
    data_file_name = examples.download_file(
        "elbow1.dat.h5", "pyfluent/file_session", return_without_path=False
    )
    file_session = FileSession()
    file_session.read_case(case_file_name)
    file_session.read_data(data_file_name)

    data_file = file_session._data_file
    assert data_file.case_file == "elbow1.cas.h5"
    assert data_file.get_phases() == ["phase-1"]
    assert len(data_file.get_face_variables("phase-1")) == 30
    assert len(data_file.get_cell_variables("phase-1")) == 14

    assert data_file.get_cell_variables("phase-1") == [
        "SV_BF_V",
        "SV_D",
        "SV_DENSITY",
        "SV_H",
        "SV_K",
        "SV_LORENTZ_FORCE",
        "SV_MU_LAM",
        "SV_MU_T",
        "SV_P",
        "SV_T",
        "SV_U",
        "SV_V",
        "SV_W",
        "",
    ]

    assert len(data_file.get_face_scalar_field_data("phase-1", "SV_DENSITY", 3)) == 3630


def test_data_reader_multi_phase():
    case_file_name = examples.download_file(
        "mixing_elbow_mul_ph.cas.h5",
        "pyfluent/file_session",
        return_without_path=False,
    )
    data_file_name = examples.download_file(
        "mixing_elbow_mul_ph.dat.h5",
        "pyfluent/file_session",
        return_without_path=False,
    )
    file_session = FileSession()
    file_session.read_case(case_file_name)
    file_session.read_data(data_file_name)

    data_file = file_session._data_file
    assert data_file.case_file == "mixing_elbow_mul_ph.cas.h5"
    assert data_file.get_phases() == [
        "phase-1",
        "phase-2",
        "phase-3",
        "phase-4",
    ]
    assert len(data_file.get_face_variables("phase-1")) == 23
    assert len(data_file.get_face_variables("phase-3")) == 13
    assert len(data_file.get_cell_variables("phase-2")) == 14

    assert data_file.get_cell_variables("phase-2") == [
        "SV_BF_V",
        "SV_DENSITY",
        "SV_DENSITY_M1",
        "SV_MU_LAM",
        "SV_MU_T",
        "SV_U",
        "SV_U_M1",
        "SV_V",
        "SV_VOF",
        "SV_VOF_M1",
        "SV_V_M1",
        "SV_W",
        "SV_W_M1",
        "",
    ]

    assert len(data_file.get_face_scalar_field_data("phase-1", "SV_DENSITY", 33)) == 268


def test_transaction_request_single_phase():
    case_file_name = examples.download_file(
        "elbow1.cas.h5", "pyfluent/file_session", return_without_path=False
    )
    data_file_name = examples.download_file(
        "elbow1.dat.h5", "pyfluent/file_session", return_without_path=False
    )
    file_session = FileSession()
    file_session.read_case(case_file_name)
    file_session.read_data(data_file_name)

    field_data = file_session.field_data

    transaction_1 = field_data.new_transaction()

    transaction_1.add_surfaces_request([3, 5])

    transaction_1.add_scalar_fields_request("SV_T", [3, 5])
    transaction_1.add_scalar_fields_request("SV_T", surface_names=["wall", "symmetry"])

    transaction_1.add_vector_fields_request("velocity", [3, 5])

    data = transaction_1.get_fields()

    assert data

    assert len(data) == 3

    # Surfaces Data
    surface_data = data[(("type", "surface-data"),)]
    assert len(surface_data) == 2
    assert list(surface_data[3].keys()) == ["faces", "vertices"]

    # Scalar Field Data
    scalar_field_tag = (
        ("type", "scalar-field"),
        ("dataLocation", 1),
        ("boundaryValues", False),
    )
    scalar_data = data[scalar_field_tag]
    assert len(scalar_data) == 3
    assert len(scalar_data[5]["SV_T"]) == 100
    assert round(scalar_data[5]["SV_T"][50], 2) == 295.43

    surf_id = file_session.field_info.get_surfaces_info()["symmetry"]["surface_id"][0]
    assert round(scalar_data[surf_id]["SV_T"][50], 2) == 293.15

    # Vector Field Data
    vector_data = data[(("type", "vector-field"),)]
    assert len(vector_data) == 2
    assert len(vector_data[5]["velocity"]) == 300
    assert round(vector_data[5]["velocity"][150], 5) == 0.03035
    assert round(vector_data[5]["velocity"][151], 5) == 0.49224
    assert round(vector_data[5]["velocity"][152], 5) == 0.00468


def test_transaction_request_multi_phase():
    case_file_name = examples.download_file(
        "mixing_elbow_mul_ph.cas.h5",
        "pyfluent/file_session",
        return_without_path=False,
    )
    data_file_name = examples.download_file(
        "mixing_elbow_mul_ph.dat.h5",
        "pyfluent/file_session",
        return_without_path=False,
    )
    file_session = FileSession()
    file_session.read_case(case_file_name)
    file_session.read_data(data_file_name)

    field_data = file_session.field_data

    transaction_1 = field_data.new_transaction()

    transaction_1.add_scalar_fields_request("phase-2:SV_WALL_YPLUS", [29, 30])

    transaction_1.add_vector_fields_request("phase-3:velocity", [31])

    data = transaction_1.get_fields()

    assert data

    assert len(data) == 2

    # Scalar Field Data
    scalar_field_tag = (
        ("type", "scalar-field"),
        ("dataLocation", 1),
        ("boundaryValues", False),
    )

    assert len(data[scalar_field_tag]) == 2

    # Vector Field Data
    vector_data = data[(("type", "vector-field"),)]
    assert len(vector_data) == 1
    assert len(vector_data[31]["phase-3:velocity"]) == 456


def test_error_handling_single_phase():
    case_file_name = examples.download_file(
        "elbow1.cas.h5", "pyfluent/file_session", return_without_path=False
    )
    data_file_name = examples.download_file(
        "elbow1.dat.h5", "pyfluent/file_session", return_without_path=False
    )
    file_session = FileSession()
    file_session.read_case(case_file_name)
    file_session.read_data(data_file_name)

    field_data = file_session.field_data

    transaction_1 = field_data.new_transaction()

    with pytest.raises(NotImplementedError) as msg:
        transaction_1.add_pathlines_fields_request("SV_T", [3, 5])

    with pytest.raises(NotImplementedError) as msg:
        field_data.get_pathlines_field_data("SV_T", [3, 5])


def test_error_handling_multi_phase():
    case_file_name = examples.download_file(
        "mixing_elbow_mul_ph.cas.h5",
        "pyfluent/file_session",
        return_without_path=False,
    )
    data_file_name = examples.download_file(
        "mixing_elbow_mul_ph.dat.h5",
        "pyfluent/file_session",
        return_without_path=False,
    )
    file_session = FileSession()
    file_session.read_case(case_file_name)
    file_session.read_data(data_file_name)

    field_data = file_session.field_data

    transaction_1 = field_data.new_transaction()
    with pytest.raises(InvalidMultiPhaseFieldName) as msg:
        transaction_1.add_scalar_fields_request("SV_WALL_YPLUS", [29, 30])

    with pytest.raises(InvalidMultiPhaseFieldName) as msg:
        d_size = field_data.get_vector_field_data("velocity", surface_ids=[34])[34].size

    with pytest.raises(InvalidFieldName) as msg:
        d_size = field_data.get_vector_field_data(
            "phase-1:temperature", surface_ids=[34]
        )[34].size

    with pytest.raises(SurfaceSpecificationError) as msg:
        d_size = field_data.get_vector_field_data(
            "velocity", surface_ids=[34], surface_name="wall"
        )[34].size



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .option_7 import option as option_cls
from .value_3 import value as value_cls
from .piecewise_linear_1 import piecewise_linear as piecewise_linear_cls
from .piecewise_polynomial import piecewise_polynomial as piecewise_polynomial_cls
from .polynomial_1 import polynomial as polynomial_cls
from .expression import expression as expression_cls
from .power_law import power_law as power_law_cls
from .blottner_curve_fit import blottner_curve_fit as blottner_curve_fit_cls
from .gupta_curve_fit_viscosity import gupta_curve_fit_viscosity as gupta_curve_fit_viscosity_cls
from .sutherland import sutherland as sutherland_cls
from .cross import cross as cross_cls
from .herschel_bulkley import herschel_bulkley as herschel_bulkley_cls
from .carreau import carreau as carreau_cls
from .non_newtonian_power_law import non_newtonian_power_law as non_newtonian_power_law_cls
from .user_defined_function import user_defined_function as user_defined_function_cls
from .rgp_table import rgp_table as rgp_table_cls
from .real_gas_nist import real_gas_nist as real_gas_nist_cls
class viscosity(Group):
    """
    'viscosity' child.
    """

    fluent_name = "viscosity"

    child_names = \
        ['option', 'value', 'piecewise_linear', 'piecewise_polynomial',
         'polynomial', 'expression', 'power_law', 'blottner_curve_fit',
         'gupta_curve_fit_viscosity', 'sutherland', 'cross',
         'herschel_bulkley', 'carreau', 'non_newtonian_power_law',
         'user_defined_function', 'rgp_table', 'real_gas_nist']

    option: option_cls = option_cls
    """
    option child of viscosity.
    """
    value: value_cls = value_cls
    """
    value child of viscosity.
    """
    piecewise_linear: piecewise_linear_cls = piecewise_linear_cls
    """
    piecewise_linear child of viscosity.
    """
    piecewise_polynomial: piecewise_polynomial_cls = piecewise_polynomial_cls
    """
    piecewise_polynomial child of viscosity.
    """
    polynomial: polynomial_cls = polynomial_cls
    """
    polynomial child of viscosity.
    """
    expression: expression_cls = expression_cls
    """
    expression child of viscosity.
    """
    power_law: power_law_cls = power_law_cls
    """
    power_law child of viscosity.
    """
    blottner_curve_fit: blottner_curve_fit_cls = blottner_curve_fit_cls
    """
    blottner_curve_fit child of viscosity.
    """
    gupta_curve_fit_viscosity: gupta_curve_fit_viscosity_cls = gupta_curve_fit_viscosity_cls
    """
    gupta_curve_fit_viscosity child of viscosity.
    """
    sutherland: sutherland_cls = sutherland_cls
    """
    sutherland child of viscosity.
    """
    cross: cross_cls = cross_cls
    """
    cross child of viscosity.
    """
    herschel_bulkley: herschel_bulkley_cls = herschel_bulkley_cls
    """
    herschel_bulkley child of viscosity.
    """
    carreau: carreau_cls = carreau_cls
    """
    carreau child of viscosity.
    """
    non_newtonian_power_law: non_newtonian_power_law_cls = non_newtonian_power_law_cls
    """
    non_newtonian_power_law child of viscosity.
    """
    user_defined_function: user_defined_function_cls = user_defined_function_cls
    """
    user_defined_function child of viscosity.
    """
    rgp_table: rgp_table_cls = rgp_table_cls
    """
    rgp_table child of viscosity.
    """
    real_gas_nist: real_gas_nist_cls = real_gas_nist_cls
    """
    real_gas_nist child of viscosity.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#


from typing import Union, List, Tuple

class geko_options:
    fluent_name = ...
    child_names = ...
    wall_distance_free = ...
    cjet = ...
    creal = ...
    cnw_sub = ...
    cjet_aux = ...
    cbf_lam = ...
    cbf_tur = ...
    command_names = ...

    def geko_defaults(self, ):
        """
        Set GEKO options to default.
        """




#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .use import use as use_cls
from .auto_2 import auto as auto_cls
from .threshold import threshold as threshold_cls
from .interval import interval as interval_cls
class dynamic_mesh(Group):
    """
    Use load balancing for dynamic mesh?.
    """

    fluent_name = "dynamic-mesh"

    child_names = \
        ['use', 'auto', 'threshold', 'interval']

    use: use_cls = use_cls
    """
    use child of dynamic_mesh.
    """
    auto: auto_cls = auto_cls
    """
    auto child of dynamic_mesh.
    """
    threshold: threshold_cls = threshold_cls
    """
    threshold child of dynamic_mesh.
    """
    interval: interval_cls = interval_cls
    """
    interval child of dynamic_mesh.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .name import name as name_cls
from .attribute import attribute as attribute_cls
from .value_1 import value as value_cls
from .display_3 import display as display_cls
class quadric_surface_child(Group):
    """
    'child_object_type' of quadric_surface.
    """

    fluent_name = "child-object-type"

    child_names = \
        ['name', 'attribute', 'value']

    name: name_cls = name_cls
    """
    name child of quadric_surface_child.
    """
    attribute: attribute_cls = attribute_cls
    """
    attribute child of quadric_surface_child.
    """
    value: value_cls = value_cls
    """
    value child of quadric_surface_child.
    """
    command_names = \
        ['display']

    display: display_cls = display_cls
    """
    display command of quadric_surface_child.
    """



import pytest
from util.solver_workflow import new_solver_session  # noqa: F401

from ansys.fluent.core.examples import download_file


@pytest.mark.nightly
@pytest.mark.fluent_version(">=23.1")
def test_setup_models_viscous_model_settings(new_solver_session) -> None:
    solver_session = new_solver_session
    case_path = download_file("elbow_source_terms.cas.h5", "pyfluent/mixing_elbow")
    solver_session.file.read(
        file_name=case_path, file_type="case", lightweight_setup=True
    )
    # NOTE: Not sure why initialization is necessary here
    # solver_session.solution.initialization.hybrid_initialize()

    viscous_model = solver_session.setup.models.viscous

    assert viscous_model.model() == "k-epsilon"
    assert "inviscid" in viscous_model.model.get_attr("allowed-values")
    viscous_model.model = "inviscid"

    assert viscous_model.model() == "inviscid"


@pytest.mark.nightly
@pytest.mark.fluent_version(">=24.1")
def test_wildcard(new_solver_session):
    solver = new_solver_session
    case_path = download_file("elbow_source_terms.cas.h5", "pyfluent/mixing_elbow")
    solver.file.read(file_name=case_path, file_type="case", lightweight_setup=True)
    boundary_conditions = solver.setup.boundary_conditions
    assert boundary_conditions.velocity_inlet["inl*"].momentum.velocity() == {
        "inlet2": {"momentum": {"velocity": {"option": "value", "value": 15}}},
        "inlet1": {"momentum": {"velocity": {"option": "value", "value": 5}}},
    }
    assert boundary_conditions.velocity_inlet["inl*"].momentum.velocity.value() == {
        "inlet2": {"momentum": {"velocity": {"value": 15}}},
        "inlet1": {"momentum": {"velocity": {"value": 5}}},
    }
    boundary_conditions.velocity_inlet["inl*"].momentum.velocity = 10
    assert boundary_conditions.velocity_inlet["inl*"].momentum.velocity() == {
        "inlet2": {"momentum": {"velocity": {"option": "value", "value": 10}}},
        "inlet1": {"momentum": {"velocity": {"option": "value", "value": 10}}},
    }
    boundary_conditions.velocity_inlet = boundary_conditions.velocity_inlet[
        "inl*"
    ].momentum.velocity()
    assert boundary_conditions.velocity_inlet["inl*"].momentum.velocity() == {
        "inlet2": {"momentum": {"velocity": {"option": "value", "value": 10}}},
        "inlet1": {"momentum": {"velocity": {"option": "value", "value": 10}}},
    }
    cell_zone_conditions = solver.setup.cell_zone_conditions
    assert cell_zone_conditions.fluid["*"].source_terms.source_terms["*mom*"]() == {
        "fluid": {
            "source_terms": {
                "source_terms": {
                    "x-momentum": [{"option": "value", "value": 1}],
                    "y-momentum": [{"option": "value", "value": 2}],
                    "z-momentum": [{"option": "value", "value": 3}],
                }
            }
        }
    }
    cell_zone_conditions.fluid["*"].source_terms.source_terms["*mom*"] = [
        {"option": "value", "value": 2}
    ]
    assert cell_zone_conditions.fluid["*"].source_terms.source_terms["*mom*"]() == {
        "fluid": {
            "source_terms": {
                "source_terms": {
                    "x-momentum": [{"option": "value", "value": 2}],
                    "y-momentum": [{"option": "value", "value": 2}],
                    "z-momentum": [{"option": "value", "value": 2}],
                }
            }
        }
    }


@pytest.mark.nightly
@pytest.mark.fluent_version(">=23.2")
def test_wildcard_fnmatch(new_solver_session):
    solver = new_solver_session
    case_path = download_file("elbow_source_terms.cas.h5", "pyfluent/mixing_elbow")
    solver.file.read_case(file_name=case_path)

    solver.solution.initialization.hybrid_initialize()

    mesh = solver.results.graphics.mesh
    mesh.create("mesh-2")
    mesh.create("mesh-a")
    mesh.create("mesh-bc")

    assert sorted(mesh["mesh-*"]()) == sorted(["mesh-1", "mesh-2", "mesh-a", "mesh-bc"])

    assert list(mesh["mesh-?c"]().keys()) == ["mesh-bc"]

    assert list(mesh["mesh-[2-5]"]().keys()) == ["mesh-2"]

    assert sorted(mesh["mesh-[!2-5]"]()) == sorted(["mesh-1", "mesh-a"])


@pytest.mark.nightly
@pytest.mark.fluent_version(">=23.2")
def test_wildcard_path_is_iterable(new_solver_session):
    solver = new_solver_session
    case_path = download_file("elbow_source_terms.cas.h5", "pyfluent/mixing_elbow")
    solver.file.read(file_name=case_path, file_type="case", lightweight_setup=True)

    velocity_inlet = solver.setup.boundary_conditions.velocity_inlet
    assert [x for x in velocity_inlet] == ["inlet2", "inlet1"]
    assert [x for x in velocity_inlet["*let*"]] == ["inlet2", "inlet1"]
    assert [x for x in velocity_inlet["*1*"]] == ["inlet1"]

    test_data = []
    for k, v in velocity_inlet.items():
        test_data.append((k, v))

    assert test_data[0][0] == "inlet2"
    assert test_data[0][1].path == r"setup/boundary-conditions/velocity-inlet/inlet2"
    assert test_data[1][0] == "inlet1"
    assert test_data[1][1].path == r"setup/boundary-conditions/velocity-inlet/inlet1"

    test_data = []
    for k, v in velocity_inlet["*let*"].items():
        test_data.append((k, v))

    assert test_data[0][0] == "inlet2"
    assert test_data[0][1].path == r"setup/boundary-conditions/velocity-inlet/inlet2"
    assert test_data[1][0] == "inlet1"
    assert test_data[1][1].path == r"setup/boundary-conditions/velocity-inlet/inlet1"


@pytest.mark.fluent_version(">=23.1")
def test_api_upgrade(new_solver_session, capsys):
    solver = new_solver_session
    case_path = download_file("Static_Mixer_main.cas.h5", "pyfluent/static_mixer")
    solver.tui.file.read_case(case_path)
    "<solver_session>.file.read_case" in capsys.readouterr().out



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .model_2 import model as model_cls
from .options_3 import options as options_cls
from .reactions import reactions as reactions_cls
from .wall_surface_options import wall_surface_options as wall_surface_options_cls
from .turb_chem_interaction_model import turb_chem_interaction_model as turb_chem_interaction_model_cls
from .turb_chem_interaction_model_options import turb_chem_interaction_model_options as turb_chem_interaction_model_options_cls
from .species_transport_expert_options import species_transport_expert_options as species_transport_expert_options_cls
from .edc_model_options import edc_model_options as edc_model_options_cls
from .tfm_model_options import tfm_model_options as tfm_model_options_cls
from .chemistry_solver import chemistry_solver as chemistry_solver_cls
from .integration_parameters import integration_parameters as integration_parameters_cls
class species(Group):
    """
    'species' child.
    """

    fluent_name = "species"

    child_names = \
        ['model', 'options', 'reactions', 'wall_surface_options',
         'turb_chem_interaction_model', 'turb_chem_interaction_model_options',
         'species_transport_expert_options', 'edc_model_options',
         'tfm_model_options', 'chemistry_solver', 'integration_parameters']

    model: model_cls = model_cls
    """
    model child of species.
    """
    options: options_cls = options_cls
    """
    options child of species.
    """
    reactions: reactions_cls = reactions_cls
    """
    reactions child of species.
    """
    wall_surface_options: wall_surface_options_cls = wall_surface_options_cls
    """
    wall_surface_options child of species.
    """
    turb_chem_interaction_model: turb_chem_interaction_model_cls = turb_chem_interaction_model_cls
    """
    turb_chem_interaction_model child of species.
    """
    turb_chem_interaction_model_options: turb_chem_interaction_model_options_cls = turb_chem_interaction_model_options_cls
    """
    turb_chem_interaction_model_options child of species.
    """
    species_transport_expert_options: species_transport_expert_options_cls = species_transport_expert_options_cls
    """
    species_transport_expert_options child of species.
    """
    edc_model_options: edc_model_options_cls = edc_model_options_cls
    """
    edc_model_options child of species.
    """
    tfm_model_options: tfm_model_options_cls = tfm_model_options_cls
    """
    tfm_model_options child of species.
    """
    chemistry_solver: chemistry_solver_cls = chemistry_solver_cls
    """
    chemistry_solver child of species.
    """
    integration_parameters: integration_parameters_cls = integration_parameters_cls
    """
    integration_parameters child of species.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .list_properties import list_properties as list_properties_cls
from .beach_dir_list_child import beach_dir_list_child

class beach_dir_list(ListObject[beach_dir_list_child]):
    """
    Multi-Directional Beach Inputs.
    """

    fluent_name = "beach-dir-list"

    command_names = \
        ['list_properties']

    list_properties: list_properties_cls = list_properties_cls
    """
    list_properties command of beach_dir_list.
    """
    child_object_type: beach_dir_list_child = beach_dir_list_child
    """
    child_object_type of beach_dir_list.
    """



#
# This is an auto-generated file.  DO NOT EDIT!
#


from typing import Union, List, Tuple

class views:
    fluent_name = ...
    child_names = ...
    camera = ...
    display_states = ...
    rendering_options = ...
    command_names = ...

    def auto_scale(self, ):
        """
        'auto_scale' command.
        """

    def reset_to_default_view(self, ):
        """
        Reset view to front and center.
        """

    def delete_view(self, view_name: str):
        """
        Remove a view from the list.
        
        Parameters
        ----------
            view_name : str
                'view_name' child.
        
        """

    def last_view(self, ):
        """
        Return to the camera position before the last manipulation.
        """

    def next_view(self, ):
        """
        Return to the camera position after the current position in the stack.
        """

    def list_views(self, ):
        """
        List predefined and saved views.
        """

    def restore_view(self, view_name: str):
        """
        Use a saved view.
        
        Parameters
        ----------
            view_name : str
                'view_name' child.
        
        """

    def read_views(self, filename: str):
        """
        Read views from a view file.
        
        Parameters
        ----------
            filename : str
                'filename' child.
        
        """

    def save_view(self, view_name: str):
        """
        Save the current view to the view list.
        
        Parameters
        ----------
            view_name : str
                'view_name' child.
        
        """

    def write_views(self, file_name: str, view_list: List[str]):
        """
        Write selected views to a view file.
        
        Parameters
        ----------
            file_name : str
                'file_name' child.
            view_list : typing.List[str]
                'view_list' child.
        
        """

